def is_nonnegative_integer_equivalent_number(argument):
    """
    Is true when ``argument`` is a nonnegative integer-equivalent number.

    ..  container:: example

        >>> duration = abjad.Duration(4, 2)
        >>> abjad.mathtools.is_nonnegative_integer_equivalent_number(duration)
        True

    Returns true or false.
    """
    from abjad import mathtools
    return mathtools.is_integer_equivalent_number(argument) and 0 <= argument



import collections
import numbers
import typing
from abjad.system.AbjadValueObject import AbjadValueObject


class NoteMaker(AbjadValueObject):
    r"""
    Note-maker.

    Makes notes according to ``pitches`` and ``durations``.

    ..  container:: example

        Cycles through ``pitches`` when the length of ``pitches`` is less than
        the length of ``durations``:

        >>> maker = abjad.NoteMaker()
        >>> notes = maker([0], [(1, 16), (1, 8), (1, 8)])
        >>> staff = abjad.Staff(notes)
        >>> abjad.show(staff) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(staff)
            \new Staff
            {
                c'16
                c'8
                c'8
            }

    ..  container:: example

        Cycles through ``durations`` when the length of ``durations`` is less
        than the length of ``pitches``:

        >>> maker = abjad.NoteMaker()
        >>> notes = maker(
        ...     [0, 2, 4, 5, 7],
        ...     [(1, 16), (1, 8), (1, 8)],
        ...     )
        >>> staff = abjad.Staff(notes)
        >>> abjad.show(staff) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(staff)
            \new Staff
            {
                c'16
                d'8
                e'8
                f'16
                g'8
            }

    ..  container:: example

        Creates ad hoc tuplets for nonassignable durations:

        >>> maker = abjad.NoteMaker()
        >>> notes = maker([0], [(1, 16), (1, 12), (1, 8)])
        >>> staff = abjad.Staff(notes)
        >>> abjad.show(staff) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(staff)
            \new Staff
            {
                c'16
                \tweak edge-height #'(0.7 . 0)
                \times 2/3 {
                    c'8
                }
                c'8
            }

    ..  container:: example

        Set ``decrease_monotonic=True`` to express tied values in decreasing
        duration:

        >>> maker = abjad.NoteMaker(decrease_monotonic=True)
        >>> notes = maker([0], [(13, 16)])
        >>> staff = abjad.Staff(notes)
        >>> abjad.show(staff) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(staff)
            \new Staff
            {
                c'2.
                ~
                c'16
            }

    ..  container:: example

        Set ``decrease_monotonic=False`` to express tied values in increasing
        duration:

        >>> maker = abjad.NoteMaker(decrease_monotonic=False)
        >>> notes = maker([0], [(13, 16)])
        >>> staff = abjad.Staff(notes)
        >>> abjad.show(staff) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(staff)
            \new Staff
            {
                c'16
                ~
                c'2.
            }

    ..  container:: example

        Uses repeat ties:

        >>> maker = abjad.NoteMaker(repeat_ties=True)
        >>> notes = maker([0], [(13, 16)])
        >>> staff = abjad.Staff(notes)
        >>> abjad.show(staff) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(staff)
            \new Staff
            {
                c'2.
                c'16
                \repeatTie
            }

    ..  container:: example

        Works with pitch segments:

        >>> maker = abjad.NoteMaker()
        >>> segment = abjad.PitchSegment([-2, -1.5, 6, 7, -1.5, 7])
        >>> notes = maker(segment, [(1, 8)])
        >>> staff = abjad.Staff(notes)
        >>> abjad.show(staff) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(staff)
            \new Staff
            {
                bf8
                bqf8
                fs'8
                g'8
                bqf8
                g'8
            }

    Set ``pitches`` to a single pitch or a sequence of pitches.

    Set ``durations`` to a single duration or a list of durations.

    Returns selection.
    """

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Makers'

    __slots__ = (
        '_decrease_monotonic',
        '_repeat_ties',
        )

    _publish_storage_format = True

    ### INITIALIZER ###

    def __init__(
        self,
        decrease_monotonic=True,
        repeat_ties=False,
        ):
        self._decrease_monotonic = decrease_monotonic
        self._repeat_ties = repeat_ties

    ### SPECIAL METHODS ###

    def __call__(self, pitches, durations):
        """
        Calls note-maker on ``pitches`` and ``durations``.

        Returns selection.
        """
        import abjad
        if isinstance(pitches, str):
            pitches = pitches.split()
        if not isinstance(pitches, collections.Iterable):
            pitches = [pitches]
        if isinstance(durations, (numbers.Number, tuple)):
            durations = [durations]
        nonreduced_fractions = [abjad.NonreducedFraction(_) for _ in durations]
        size = max(len(nonreduced_fractions), len(pitches))
        nonreduced_fractions = abjad.sequence(nonreduced_fractions)
        nonreduced_fractions = nonreduced_fractions.repeat_to_length(size)
        pitches = abjad.sequence(pitches).repeat_to_length(size)
        Duration = abjad.Duration
        durations = Duration._group_by_implied_prolation(
            nonreduced_fractions)
        result = []
        for duration in durations:
            # get factors in denominator of duration group duration not 1 or 2
            factors = set(abjad.mathtools.factors(duration[0].denominator))
            factors.discard(1)
            factors.discard(2)
            ps = pitches[0:len(duration)]
            pitches = pitches[len(duration):]
            if len(factors) == 0:
                result.extend(
                    self._make_unprolated_notes(
                        ps,
                        duration,
                        decrease_monotonic=self.decrease_monotonic,
                        repeat_ties=self.repeat_ties,
                        )
                    )
            else:
                # compute prolation
                denominator = duration[0].denominator
                numerator = abjad.mathtools.greatest_power_of_two_less_equal(
                    denominator)
                multiplier = (numerator, denominator)
                ratio = 1 / abjad.Fraction(*multiplier)
                duration = [ratio * abjad.Duration(d) for d in duration]
                ns = self._make_unprolated_notes(
                    ps,
                    duration,
                    decrease_monotonic=self.decrease_monotonic,
                    repeat_ties=self.repeat_ties,
                    )
                tuplet = abjad.Tuplet(multiplier, ns)
                result.append(tuplet)
        result = abjad.select(result)
        return result

    ### PRIVATE METHODS ###

    @staticmethod
    def _make_unprolated_notes(
        pitches,
        durations,
        decrease_monotonic=True,
        repeat_ties=False,
        ):
        import abjad
        assert len(pitches) == len(durations)
        result = []
        for pitch, duration in zip(pitches, durations):
            result.extend(
                abjad.LeafMaker._make_tied_leaf(
                    abjad.Note,
                    duration,
                    pitches=pitch,
                    decrease_monotonic=decrease_monotonic,
                    repeat_ties=repeat_ties,
                    )
                )
        return result

    ### PUBLIC PROPERTIES ###

    @property
    def decrease_monotonic(self):
        """
        Is true when durations decrease monotonically.

        Returns true, false or none.
        """
        return self._decrease_monotonic

    @property
    def repeat_ties(self) -> typing.Optional[bool]:
        """
        Is true when ties are repeat ties.
        """
        return self._repeat_ties



from abjad.system.AbjadObject import AbjadObject


class LilyPondGrammarGenerator(AbjadObject):
    """
    Generates a syntax skeleton from LilyPond grammar files.
    """

    ### CLASS VARIABLES ###

    __slots__ = ()

    ### SPECIAL METHODS ###

    def __call__(
        self,
        skeleton_path,
        parser_output_path,
        parser_tab_hh_path,
        ):
        """
        Calls LilyPond grammar generator.
        """
        self._write_parser_syntax_skeleton(
            self,
            skeleton_path,
            parser_output_path,
            parser_tab_hh_path,
            )

    ### PRIVATE METHODS ###

    def _extract_productions_from_parser_output(
        self,
        file_path,
        ):
        with open(file_path, 'r') as f:
            lines = f.read().split('\n')
        productions = { }
        nonterminal = None
        in_grammar = False
        for line in lines:
            text = line.strip()
            # starting and stopping
            if text == 'Terminals, with rules where they appear':
                break
            elif text == 'Grammar':
                in_grammar = True
                continue
            if not in_grammar:
                continue
            if not text:
                continue
            parts = text.split()[1:]
            if parts[0].startswith('$'):
                continue
            elif parts[0] == '|':
                right_hand = filter(lambda x: not x.startswith('$'), parts[1:])
                productions[nonterminal].append(parts[1:])
            else:
                nonterminal = parts[0][:-1]
                if nonterminal not in productions:
                    productions[nonterminal] = []
                right_hand = parts[1:]
                if right_hand[0] == '/*': # /* empty */
                    productions[nonterminal].append([])
                else:
                    right_hand = filter(lambda x: not x.startswith('$'), right_hand)
                    productions[nonterminal].append(right_hand)
        return productions

    def _extract_token_names_from_parser_tab_hh(
        self,
        file_path,
        ):
        with open(file_path, 'r') as f:
            lines = f.read().split('\n')
        token_names = { }
        in_enum = False
        for line in lines:
            text = line.strip()
            if in_enum and text == '};':
                break;
            if in_enum:
                parts = text.split(' ')
                name = parts[0]
                if parts[2].endswith(','):
                    number = int(parts[2][:-1])
                else:
                    number = int(parts[2])
                token_names[number] = name
            if text == 'enum yytokentype {':
                in_enum = True
        return token_names

    def _extract_token_values_from_parser_output(
        self,
        file_path,
        ):
        with open(file_path, 'r') as f:
            lines = f.read().split('\n')
        token_values = { }
        in_token_list = False
        for line in lines:
            text = line.strip()
            if in_token_list and text == 'Nonterminals, with rules where they appear':
                break;
            elif text == 'Terminals, with rules where they appear':
                in_token_list = True
                continue
            elif not text:
                continue
            elif not in_token_list:
                continue
            parts = text.split()
            if parts[0].isdigit():
                continue
            elif parts[0].startswith('$'):
                continue
            value = parts[0]
            number = int(parts[1][1:-1])
            token_values[number] = value
        return token_values

    def _generate_production_map(
        self,
        output_path,
        tab_hh_path,
        ):
        productions = self._extract_productions_from_parser_output(
            output_path)
        names = self._extract_token_names_from_parser_tab_hh(
            tab_hh_path)
        values = self._extract_token_values_from_parser_output(
            output_path)
        matches = self._match_token_names_with_token_values(
            names, values)
        rewrites = { }
        for nonterminal in productions:
            for rh in productions[nonterminal]:
                for i, r in enumerate(rh):
                    if r in matches:
                        rh[i] = matches[r]
                docstring = '{} : {}'.format(nonterminal, ' '.join(rh))
                for i, r in enumerate(rh):
                    if r[0] == "'" and r[-1] == "'":
                        rh[i] = 'Chr{}'.format(ord(r[-2]))
                funcname = 'p_{}__{}'.format(nonterminal, '__'.join(rh))
                rewrites[funcname] = docstring
        return rewrites

    def _match_token_names_with_token_values(
        self,
        names,
        values,
        ):
        matches = { }
        for number, value in values.items():
            if number in names:
                name = names[number]
                matches[value] = name
        return matches

    def _write_parser_syntax_skeleton(
        self,
        skeleton_path,
        parser_output_path,
        parser_tab_hh_path,
        ):
        productions = self._generate_production_map(
            parser_output_path, parser_tab_hh_path)
        with open(skeleton_path, 'w') as f:
            f.write('from abjad import *\n')
            f.write('from abjad.parser.SyntaxNode.SyntaxNode \\\n')
            f.write('    import SyntaxNode as Node\n\n\n')
            f.write('class _LilyPondSyntacticalDefinition(object):\n\n')
            f.write('    def __init__(self, client):\n')
            f.write('        self.client = client\n')
            f.write('        self.tokens = self.client.lexdef.tokens\n\n\n')
            f.write("    start_symbol = 'start_symbol'\n\n\n")
            f.write("    precedence = (\n")
            f.write("        ('nonassoc', 'COMPOSITE'),\n")
            f.write("        ('nonassoc', 'REPEAT'),\n")
            f.write("        ('nonassoc', 'ALTERNATIVE'),\n")
            f.write("        ('left', 'ADDLYRICS'),\n")
            f.write("        ('nonassoc', 'DEFAULT'),\n")
            f.write("        ('nonassoc', 'FUNCTION_ARGLIST'),\n")
            f.write("        ('right', 'PITCH_IDENTIFIER', 'NOTENAME_PITCH', 'TONICNAME_PITCH', 'UNSIGNED', 'REAL', 'DURATION_IDENTIFIER', ':'),\n")
            f.write("        ('nonassoc', 'NUMBER_IDENTIFIER', '/'),\n")
            f.write("    )\n\n\n")
            f.write('    ### SYNTACTICAL RULES (ALPHABETICAL) ###\n\n\n')
            current_nonterminal = 'start_symbol'
            ly_keys = sorted(key for key in productions
                if key.startswith('p_start_symbol'))
            for key in ly_keys:
                funcname = key
                docstring = productions[key]
                f.write('    def {}(self, p):\n'.format(funcname))
                f.write("        {!r}\n".format(docstring))
                f.write("        p[0] = Node('{}', p[1:])\n\n\n".format(
                    current_nonterminal))
            for funcname, docstring in sorted(productions.items()):
                nonterminal = funcname.split('__')[0][2:]
                if nonterminal == 'start_symbol':
                    continue
                if nonterminal != current_nonterminal:
                    current_nonterminal = nonterminal
                    f.write('    ### {} ###\n\n\n'.format(current_nonterminal))
                f.write('    def {}(self, p):\n'.format(funcname))
                f.write("        {!r}\n".format(docstring))
                f.write("        p[0] = Node('{}', p[1:])\n\n\n".format(
                    current_nonterminal))
            f.write('    def p_error(self, p):\n')
            f.write('        pass\n\n')



def difference_series(argument):
    """
    Gets difference series of ``argument``.

    ..  container:: example

        >>> abjad.mathtools.difference_series([1, 1, 2, 3, 5, 5, 6])
        [0, 1, 1, 2, 0, 1]

        >>> abjad.mathtools.difference_series([9, 6, 8, 5, 7, 4, 6])
        [-3, 2, -3, 2, -3, 2]

    Returns new object of ``argument`` type.
    """
    result = []
    for i, n in enumerate(argument[1:]):
        result.append(n - argument[i])
    return type(argument)(result)



import typing
from abjad.system.AbjadValueObject import AbjadValueObject
from abjad.markups import Markup
from abjad.system.LilyPondFormatBundle import LilyPondFormatBundle
from abjad.top.new import new


class MarginMarkup(AbjadValueObject):
    r"""
    Margin markup.

    ..  container:: example

        >>> staff = abjad.Staff("c'4 d'4 e'4 f'4")
        >>> margin_markup = abjad.MarginMarkup(
        ...     markup=abjad.Markup('Vc.'),
        ...     )
        >>> abjad.attach(margin_markup, staff[0])
        >>> abjad.show(staff, strict=89) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(staff)
            \new Staff
            {
                \set Staff.instrumentName =
                \markup { Vc. }
                \set Staff.shortInstrumentName =
                \markup { Vc. }
                c'4
                d'4
                e'4
                f'4
            }

    """

    ### CLASS VARIABLES ###

    __slots__ = (
        '_context',
        '_format_slot',
        '_markup',
        )

    _latent = True

    _parameter = True

    _publish_storage_format = True

    _redraw = True

    ### INITIALIZER ##

    def __init__(
        self,
        *,
        context: str = 'Staff',
        format_slot: str = 'before',
        markup: Markup = None,
        ) -> None:
        self._context = context
        self._format_slot = format_slot
        self._markup = markup

    ### SPECIAL METHODS ###

    def __eq__(self, argument) -> bool:
        """
        Is true when ``argument`` is margin markup with context and markup
        equal to those of this margin markup.

        ..  container:: example

            >>> margin_markup_1 = abjad.MarginMarkup(
            ...     context='PianoStaff',
            ...     markup=abjad.Markup('Hp.'),
            ...     )
            >>> margin_markup_2 = abjad.MarginMarkup(
            ...     context='PianoStaff',
            ...     markup=abjad.Markup('Hp.'),
            ...     )
            >>> margin_markup_3 = abjad.MarginMarkup(
            ...     context='Staff',
            ...     markup=abjad.Markup('Hp.'),
            ...     )

            >>> margin_markup_1 == margin_markup_1
            True
            >>> margin_markup_1 == margin_markup_2
            True
            >>> margin_markup_1 == margin_markup_3
            False

            >>> margin_markup_2 == margin_markup_1
            True
            >>> margin_markup_2 == margin_markup_2
            True
            >>> margin_markup_2 == margin_markup_3
            False

            >>> margin_markup_3 == margin_markup_1
            False
            >>> margin_markup_3 == margin_markup_2
            False
            >>> margin_markup_3 == margin_markup_3
            True

        """
        if not isinstance(argument, type(self)):
            return False
        if self.context == argument.context and self.markup == argument.markup:
            return True
        return False

    def __hash__(self) -> int:
        r"""Hashes margin markup.

        Redefined in tandem with __eq__.

        ..  container:: example

            >>> margin_markup = abjad.MarginMarkup(
            ...     context='PianoStaff',
            ...     markup=abjad.Markup('Hp.'),
            ...     )

            >>> hash_ = hash(margin_markup)
            >>> isinstance(hash_, int)
            True

        """
        return super().__hash__()

    ### PRIVATE PROPERTIES ###

    @property
    def _lilypond_type(self):
        if isinstance(self.context, type):
            return self.context.__name__
        elif isinstance(self.context, str):
            return self.context
        else:
            return type(self.context).__name__

    ### PRIVATE METHODS ###

    def _get_lilypond_format(self, context=None):
        result = []
        markup = self.markup
        if markup.direction is not None:
            markup = new(
                markup,
                direction=None,
                )
        if isinstance(context, str):
            pass
        elif context is not None:
            context = context.lilypond_type
        else:
            context = self._lilypond_type
        pieces = markup._get_format_pieces()
        result.append(rf'\set {context!s}.instrumentName =')
        result.extend(pieces)
        result.append(rf'\set {context!s}.shortInstrumentName =')
        result.extend(pieces)
        return result

    def _get_lilypond_format_bundle(self, component=None):
        bundle = LilyPondFormatBundle()
        slot = bundle.get(self.format_slot)
        slot.commands.extend(self._get_lilypond_format())
        return bundle

    ### PUBLIC PROPERTIES ###

    @property
    def context(self) -> str:
        """
        Gets default context of margin markup.

        ..  container:: example

            >>> abjad.MarginMarkup().context
            'Staff'

        """
        return self._context

    @property
    def format_slot(self) -> str:
        """
        Gets format slot.

        ..  container:: example

            >>> abjad.MarginMarkup().format_slot
            'before'

        """
        return self._format_slot

    @property
    def latent(self) -> bool:
        """Is true.

        ..  container::

            >>> margin_markup = abjad.MarginMarkup(
            ...     markup=abjad.Markup('Vc.'),
            ...     )
            >>> margin_markup.latent
            True

        Class constant.
        """
        return self._latent

    @property
    def markup(self) -> typing.Optional[Markup]:
        """
        Gets (instrument name) markup.
        """
        return self._markup

    @property
    def parameter(self) -> bool:
        """
        Is true.

        ..  container:: example

            >>> margin_markup = abjad.MarginMarkup(
            ...     markup=abjad.Markup('Vc.'),
            ...     )
            >>> margin_markup.parameter
            True

        Class constant.
        """
        return self._parameter

    @property
    def redraw(self) -> bool:
        """
        Is true.

        ..  container:: example

            >>> margin_markup = abjad.MarginMarkup(
            ...     markup=abjad.Markup('Vc.'),
            ...     )
            >>> margin_markup.redraw
            True

        Class constant.
        """
        return self._redraw

    @property
    def tweaks(self) -> None:
        """
        Are not implemented on margin markup.
        
        The LilyPond ``\shortInstrumentName`` command refuses tweaks.

        Craft explicit markup instead.
        """
        pass



import copy
from abjad.lilypondnames.LilyPondContext import LilyPondContext
from abjad.system.LilyPondFormatManager import LilyPondFormatManager
from .Container import Container


class Context(Container):
    """
    LilyPond context.

    ..  container:: example

        >>> context = abjad.Context(
        ...     name='MeterVoice',
        ...     lilypond_type='GlobalContext',
        ...     )

        >>> context
        Context(lilypond_type='GlobalContext', name='MeterVoice')

        ..  docs::

            >>> abjad.f(context)
            \context GlobalContext = "MeterVoice"
            {
            }

    """

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Contexts'

    __slots__ = (
        '_lilypond_type',
        '_consists_commands',
        '_dependent_wrappers',
        '_remove_commands',
        )

    _default_lilypond_type = 'Voice'

    lilypond_types = (
        'Score',
        'StaffGroup',
        'ChoirStaff',
        'GrandStaff',
        'PianoStaff',
        'Staff',
        'RhythmicStaff',
        'TabStaff',
        'DrumStaff',
        'VaticanaStaff',
        'MensuralStaff',
        'Voice',
        'VaticanaVoice',
        'MensuralVoice',
        'Lyrics',
        'DrumVoice',
        'FiguredBass',
        'TabVoice',
        'CueVoice',
        'ChordNames',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        components=None,
        lilypond_type='Context',
        is_simultaneous=None,
        name=None,
        ):
        self._consists_commands = []
        self._dependent_wrappers = []
        self._remove_commands = []
        self.lilypond_type = lilypond_type
        Container.__init__(
            self,
            is_simultaneous=is_simultaneous,
            components=components,
            name=name,
            )

    ### SPECIAL METHODS ###

    def __copy__(self, *arguments):
        """
        Shallow copies context.

        Copies indicators.

        Does not copy spanners.

        Does not copy children.

        Returns new component.
        """
        new_context = Container.__copy__(self)
        new_context._consists_commands = copy.copy(self.consists_commands)
        new_context._remove_commands = copy.copy(self.remove_commands)
        return new_context

    def __getnewargs__(self):
        """
        Gets new container arguments.

        Returns tuple.
        """
        return [], self.lilypond_type, self.is_simultaneous, self.name

    def __repr__(self):
        """
        Gets interpreter representation of context.

        >>> context = abjad.Context(
        ...     name='MeterVoice',
        ...     lilypond_type='GlobalContext',
        ...     )
        >>> repr(context)
        "Context(lilypond_type='GlobalContext', name='MeterVoice')"

        Returns string.
        """
        if self[:].are_leaves():
            return Container.__repr__(self)
        return self._get_abbreviated_string_format()

    ### PRIVATE METHODS ###

    def _format_closing_slot(context, bundle):
        result = []
        result.append(('indicators', bundle.closing.indicators))
        result.append(('commands', bundle.closing.commands))
        result.append(('comments', bundle.closing.comments))
        return context._format_slot_contributions_with_indent(result)

    def _format_consists_commands(self):
        result = []
        for engraver in self.consists_commands:
            string = rf'\consists {engraver}'
            result.append(string)
        return result

    def _format_invocation(self):
        if self.name is not None:
            string = rf'\context {self.lilypond_type} = "{self.name}"'
        else:
            string = rf'\new {self.lilypond_type}'
        return string

    def _format_open_brackets_slot(context, bundle):
        indent = LilyPondFormatManager.indent
        result = []
        if context.is_simultaneous:
            if context.identifier:
                open_bracket = f'<<  {context.identifier}'
            else:
                open_bracket = '<<'
        else:
            if context.identifier:
                open_bracket = f'{{   {context.identifier}'
            else:
                open_bracket = '{'
        brackets_open = [open_bracket]
        remove_commands = context._format_remove_commands()
        consists_commands = context._format_consists_commands()
        overrides = bundle.grob_overrides
        settings = bundle.context_settings
        if remove_commands or consists_commands or overrides or settings:
            contributions = [context._format_invocation(), r'\with', '{']
            contributions = tuple(contributions)
            identifier_pair = ('context_brackets', 'open')
            result.append((identifier_pair, contributions))
            contributions = [indent + _ for _ in remove_commands]
            contributions = tuple(contributions)
            identifier_pair = ('engraver removals', 'remove_commands')
            result.append((identifier_pair, contributions))
            contributions = [indent + _ for _ in consists_commands]
            contributions = tuple(contributions)
            identifier_pair = ('engraver consists', 'consists_commands')
            result.append((identifier_pair, contributions))
            contributions = [indent + _ for _ in overrides]
            contributions = tuple(contributions)
            identifier_pair = ('overrides', 'overrides')
            result.append((identifier_pair, contributions))
            contributions = [indent + _ for _ in settings]
            contributions = tuple(contributions)
            identifier_pair = ('settings', 'settings')
            result.append((identifier_pair, contributions))
            contributions = ['}} {}'.format(brackets_open[0])]
            contributions = ['}', open_bracket]
            contributions = tuple(contributions)
            identifier_pair = ('context_brackets', 'open')
            result.append((identifier_pair, contributions))
        else:
            contribution = context._format_invocation()
            contribution += ' {}'.format(brackets_open[0])
            contributions = [contribution]
            contributions = [context._format_invocation(), open_bracket]
            contributions = tuple(contributions)
            identifier_pair = ('context_brackets', 'open')
            result.append((identifier_pair, contributions))
        return tuple(result)

    def _format_opening_slot(context, bundle):
        result = []
        result.append(('comments', bundle.opening.comments))
        result.append(('indicators', bundle.opening.indicators))
        result.append(('commands', bundle.opening.commands))
        return context._format_slot_contributions_with_indent(result)

    def _format_remove_commands(self):
        result = []
        for engraver in self.remove_commands:
            string = r'\remove {}'.format(engraver)
            result.append(string)
        return result

    def _get_format_pieces(self):
        return self._format_component(pieces=True)

    def _get_lilypond_format(self):
        self._update_now(indicators=True)
        return self._format_component()

    def _get_persistent_wrappers(self):
        self._update_now(indicators=True)
        wrappers = {}
        for wrapper in self._dependent_wrappers:
            if wrapper.annotation:
                continue
            indicator = wrapper.indicator
            if not getattr(indicator, 'parameter', False):
                continue
            if isinstance(indicator.parameter, str):
                key = indicator.parameter
            else:
                key = str(type(indicator))
            if (key not in wrappers or
                wrappers[key].start_offset <= wrapper.start_offset):
                wrappers[key] = wrapper
        return wrappers

    def _get_repr_kwargs_names(self):
        if self.lilypond_type == type(self).__name__:
            return ['is_simultaneous', 'name']
        else:
            return ['is_simultaneous', 'lilypond_type', 'name']

    ### PUBLIC PROPERTIES ###

    @property
    def consists_commands(self):
        r"""
        Unordered set of LilyPond engravers to include
        in context definition.

        Manage with add, update, other standard set commands:

        >>> staff = abjad.Staff([])
        >>> staff.consists_commands.append('Horizontal_bracket_engraver')
        >>> abjad.f(staff)
        \new Staff
        \with
        {
            \consists Horizontal_bracket_engraver
        }
        {
        }

        """
        return self._consists_commands

    @property
    def lilypond_type(self):
        """
        Gets lilypond type.

        ..  container:: example

            >>> context = abjad.Context(
            ...     lilypond_type='ViolinStaff',
            ...     name='MyViolinStaff',
            ...     )
            >>> context.lilypond_type
            'ViolinStaff'
        
        Gets and sets lilypond type of context.

        Returns string.
        """
        return self._lilypond_type

    @lilypond_type.setter
    def lilypond_type(self, argument):
        if argument is None:
            argument = type(self).__name__
        else:
            argument = str(argument)
        self._lilypond_type = argument

    @property
    def lilypond_context(self):
        """
        Gets ``LilyPondContext`` associated with context.

        Returns LilyPond context instance.
        """
        try:
            lilypond_context = LilyPondContext(name=self.lilypond_type)
        except AssertionError:
            lilypond_context = LilyPondContext(
                name=self._default_lilypond_type,
                )
        return lilypond_context

    @property
    def remove_commands(self):
        r"""
        Unordered set of LilyPond engravers to remove from context.

        Manage with add, update, other standard set commands:

        >>> staff = abjad.Staff([])
        >>> staff.remove_commands.append('Time_signature_engraver')
        >>> abjad.f(staff)
        \new Staff
        \with
        {
            \remove Time_signature_engraver
        }
        {
        }

        """
        return self._remove_commands



import abc
import copy
import functools
import numbers
from abjad import mathtools
from abjad.system.AbjadValueObject import AbjadValueObject
from . import constants


@functools.total_ordering
class IntervalClass(AbjadValueObject):
    """
    Abstract interval-class.
    """

    ### CLASS VARIABLES ###

    __slots__ = ()

    ### INITIALIZER ###

    @abc.abstractmethod
    def __init__(self, argument):
        import abjad
        if isinstance(argument, str):
            match = constants._interval_name_abbreviation_regex.match(argument)
            if match is None:
                try:
                    argument = float(argument)
                    self._from_number(argument)
                    return
                except ValueError:
                    message = 'can not initialize {} from {!r}.'
                    message = message.format(type(self).__name__, argument)
                    raise ValueError(message)
                message = 'can not initialize {} from {!r}.'
                message = message.format(type(self).__name__, argument)
                raise ValueError(message)
            group_dict = match.groupdict()
            direction = group_dict['direction']
            if direction == '-':
                direction = -1
            else:
                direction = 1
            quality = group_dict['quality']
            diatonic_number = int(group_dict['number'])
            quality = self._validate_quality_and_diatonic_number(
                quality, diatonic_number,
                )
            quartertone = group_dict['quartertone']
            quality += quartertone
            self._from_named_parts(direction, quality, diatonic_number)
        elif isinstance(argument, tuple) and len(argument) == 2:
            quality, number = argument
            direction = mathtools.sign(number)
            diatonic_number = abs(number)
            quality = self._validate_quality_and_diatonic_number(
                quality, diatonic_number,
                )
            self._from_named_parts(direction, quality, diatonic_number)
        elif isinstance(argument, numbers.Number):
            self._from_number(argument)
        elif isinstance(argument, (abjad.Interval, abjad.IntervalClass)):
            self._from_interval_or_interval_class(argument)
        else:
            message = 'can not initialize {} from {!r}.'
            message = message.format(type(self).__name__, argument)
            raise ValueError(message)

    ### SPECIAL METHODS ###

    def __abs__(self):
        """
        Gets absolute value of interval-class.

        Returns new interval-class.
        """
        return type(self)(abs(self._number))

    @abc.abstractmethod
    def __add__(self, argument):
        """
        Adds `argument` to interval-class.

        Returns new interval-class.
        """
        raise NotImplementedError

    def __float__(self):
        """
        Coerce to semitones as float.

        Returns float.
        """
        raise NotImplementedError

    @abc.abstractmethod
    def __lt__(self, argument):
        """
        Is true when interval-class is less than `argument`.

        Returns true or false.
        """
        raise NotImplementedError

    def __str__(self):
        """
        Gets string representation of interval-class.

        Returns string.
        """
        return str(self.number)

    @abc.abstractmethod
    def __sub__(self, argument):
        """
        Subtracts `argument` from interval-class.

        Returns new interval-class.
        """
        raise NotImplementedError

    ### PRIVATE METHODS ###

    @classmethod
    def _named_to_numbered(cls, direction, quality, diatonic_number):
        octave_number = 0
        diatonic_pc_number = abs(diatonic_number)
        while diatonic_pc_number >= 8:
            diatonic_pc_number -= 7
            octave_number += 1

        quartertone = ''
        if quality.endswith(('+', '~')):
            quality, quartertone = quality[:-1], quality[-1]

        base_quality = quality
        if base_quality == 'P' and octave_number and diatonic_pc_number == 1:
            return 12 * direction
        if len(quality) > 1:
            base_quality = quality[0]

        semitones = constants._diatonic_number_and_quality_to_semitones[
            diatonic_pc_number][base_quality]
        if base_quality == 'd':
            semitones -= (len(quality) - 1)
        elif base_quality == 'A':
            semitones += (len(quality) - 1)

        if quartertone == '+':
            semitones += 0.5
        elif quartertone == '~':
            semitones -= 0.5

        if abs(diatonic_number) == 1:
            semitones = abs(semitones)
        while abs(semitones) > 12:
            semitones = (abs(semitones) - 12) * mathtools.sign(semitones)
        semitones *= direction
        return mathtools.integer_equivalent_number_to_integer(semitones)

    @classmethod
    def _numbered_to_named(cls, number):
        number = cls._to_nearest_quarter_tone(float(number))
        direction = mathtools.sign(number)
        octaves, semitones = divmod(abs(number), 12)
        if semitones == 0 and octaves:
            semitones = 12
        quartertone = ''
        if semitones % 1:
            semitones -= 0.5
            quartertone = '+'
        quality, diatonic_pc_number = constants._semitones_to_quality_and_diatonic_number[semitones]
        quality += quartertone
        diatonic_pc_number = cls._to_nearest_quarter_tone(diatonic_pc_number)
        return direction, quality, diatonic_pc_number

    @staticmethod
    def _to_nearest_quarter_tone(number):
        number = round(float(number) * 4) / 4
        div, mod = divmod(number, 1)
        if mod == 0.75:
            div += 1
        elif mod == 0.5:
            div += 0.5
        return mathtools.integer_equivalent_number_to_integer(div)

    @classmethod
    def _validate_quality_and_diatonic_number(cls, quality, diatonic_number):
        if quality in constants._quality_string_to_quality_abbreviation:
            quality = constants._quality_string_to_quality_abbreviation[quality]
        if quality == 'aug':
            quality = 'A'
        if quality == 'dim':
            quality = 'd'
        octaves = 0
        diatonic_pc_number = diatonic_number
        while diatonic_pc_number > 7:
            diatonic_pc_number -= 7
            octaves += 1
        if constants._diatonic_number_and_quality_to_semitones.get(
            diatonic_pc_number, {}).get(quality[0]) is None:
            message = 'can not initialize {} from {!r} and {!r}.'
            message = message.format(cls.__name__, quality, diatonic_number)
            raise ValueError(message)
        return quality

    ### PUBLIC PROPERTIES ###

    @property
    def number(self):
        """
        Gets number of interval-class.

        Returns number.
        """
        return self._number

    ### PUBLIC METHODS ###

    def transpose(self, pitch_carrier):
        """
        Transposes `pitch_carrier` by interval-class.

        Returns new pitch carrier.
        """
        import abjad
        if isinstance(pitch_carrier, (abjad.Pitch, abjad.PitchClass)):
            return pitch_carrier.transpose(self)
        elif isinstance(pitch_carrier, abjad.Note):
            new_note = copy.copy(pitch_carrier)
            new_pitch = pitch_carrier.written_pitch.transpose(self)
            new_note.written_pitch = new_pitch
            return new_note
        elif isinstance(pitch_carrier, abjad.Chord):
            new_chord = copy.copy(pitch_carrier)
            pairs = zip(new_chord.note_heads, pitch_carrier.note_heads)
            for new_nh, old_nh in pairs:
                new_pitch = old_nh.written_pitch.transpose(self)
                new_nh.written_pitch = new_pitch
            return new_chord
        return pitch_carrier



from .ContextManager import ContextManager


class NullContextManager(ContextManager):
    """
    A context manager that does nothing.
    """

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Context managers'

    __slots__ = ()

    ### INITIALIZER ###

    def __init__(self):
        pass

    ### SPECIAL METHODS ###

    def __enter__(self):
        """
        Enters context manager and does nothing.
        """
        pass

    def __exit__(self, exc_type, exc_value, traceback):
        """
        Exits context manager and does nothing.
        """
        pass



from .Container import Container


class AfterGraceContainer(Container):
    r"""
    After grace container.

    ..  container:: example

        After grace notes:

        >>> voice = abjad.Voice("c'4 d'4 e'4 f'4")
        >>> string = '#(define afterGraceFraction (cons 15 16))'
        >>> literal = abjad.LilyPondLiteral(string)
        >>> abjad.attach(literal, voice[0])
        >>> notes = [abjad.Note("c'16"), abjad.Note("d'16")]
        >>> after_grace_container = abjad.AfterGraceContainer(notes)
        >>> abjad.attach(after_grace_container, voice[1])
        >>> abjad.show(voice) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(voice)
            \new Voice
            {
                #(define afterGraceFraction (cons 15 16))
                c'4
                \afterGrace
                d'4
                {
                    c'16
                    d'16
                }
                e'4
                f'4
            }

        LilyPond positions after grace notes at a point 3/4 of the way
        after the note they follow. The resulting spacing is usually too
        loose.

        Customize aftterGraceFraction as shown above.

    After grace notes are played at the very end of the note they follow.

    Use after grace notes when you need to end a piece of music with grace
    notes.

    After grace notes do not subclass grace notes; but acciacatura containers
    and appoggiatura containers do subclass grace notes.

    Fill grace containers with notes, rests or chords.

    Attach after grace containers to notes, rests or chords.
    """

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Containers'

    __slots__ = (
        '_carrier',
        )

    ### INITIALIZER ###

    def __init__(self, components=None):
        self._carrier = None
        Container.__init__(self, components)

    ### SPECIAL METHODS ###

    def __getnewargs__(self):
        """
        Gets new after grace container arguments.

        Returns tuple of single empty list.
        """
        return ([],)

    ### PRIVATE METHODS ###

    def _attach(self, leaf):
        import abjad
        if not isinstance(leaf, abjad.Leaf):
            message = 'must attach to leaf: {!r}.'
            message = message.format(leaf)
            raise TypeError(message)
        leaf._after_grace_container = self
        self._carrier = leaf

    def _detach(self):
        if self._carrier is not None:
            carrier = self._carrier
            carrier._after_grace_container = None
            self._carrier = None
        return self

    def _format_open_brackets_slot(self, bundle):
        result = []
        result.append([('grace_brackets', 'open'), ['{']])
        return tuple(result)



import os
import shutil
import subprocess


def compare_images(image_one, image_two):
    r'''Compare `image_one` against `image_two` using ImageMagick's `compare`
    commandline tool.

    Returns true if images are the same.

    Returns false if `compare` is not available.
    '''
    import abjad
    import tempfile

    assert os.path.exists(image_one)
    assert os.path.exists(image_two)

    result = False

    if abjad.IOManager.find_executable('compare'):

        tempdir = tempfile.mkdtemp()
        comparison = os.path.join(tempdir, 'comparison.png')

        command = 'compare -metric ae {} {} {}'.format(
            image_one, image_two, comparison)
        process = subprocess.Popen(command.split(),
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE
            )

        stderr = process.stderr.read()
        stdout = process.stdout.read()

        if stderr:
            part = stderr.split()[0]
            if part.isdigit():
                result = int(part) is 0
        elif stdout:
            part = stdout.split()[0]
            if part.isdigit():
                result = int(part) is 0

        shutil.rmtree(tempdir)

    return result



"""
Tools for modeling "indicators".
"""

from .Arpeggio import Arpeggio
from .Articulation import Articulation
from .BarLine import BarLine
from .BendAfter import BendAfter
from .BowContactPoint import BowContactPoint
from .BowMotionTechnique import BowMotionTechnique
from .BowPressure import BowPressure
from .BreathMark import BreathMark
from .Clef import Clef
from .ColorFingering import ColorFingering
from .Dynamic import Dynamic
from .DynamicTrend import DynamicTrend
from .Fermata import Fermata
from .KeyCluster import KeyCluster
from .KeySignature import KeySignature
from .LaissezVibrer import LaissezVibrer
from .LilyPondComment import LilyPondComment
from .LilyPondLiteral import LilyPondLiteral
from .MarginMarkup import MarginMarkup
from .MetricModulation import MetricModulation
from .MetronomeMark import MetronomeMark
from .Mode import Mode
from .RehearsalMark import RehearsalMark
from .Repeat import Repeat
from .Staccatissimo import Staccatissimo
from .Staccato import Staccato
from .StaffChange import StaffChange
from .StartMarkup import StartMarkup
from .StartTextSpan import StartTextSpan
from .StemTremolo import StemTremolo
from .StopTextSpan import StopTextSpan
from .StringContactPoint import StringContactPoint
from .TimeSignature import TimeSignature
from .Tremolo import Tremolo
from .WoodwindFingering import WoodwindFingering



def partition_integer_by_ratio(n, ratio):
    """
    Partitions positive integer-equivalent ``n`` by ``ratio``.

    ..  container:: example

        >>> abjad.mathtools.partition_integer_by_ratio(10, [1, 2])
        [3, 7]

    ..  container:: example

        Partitions positive integer-equivalent ``n`` by ``ratio`` with negative
        parts:

        >>> abjad.mathtools.partition_integer_by_ratio(10, [1, -2])
        [3, -7]

    ..  container:: example

        Partitions negative integer-equivalent ``n`` by ``ratio``:

        >>> abjad.mathtools.partition_integer_by_ratio(-10, [1, 2])
        [-3, -7]

    ..  container:: example

        Partitions negative integer-equivalent ``n`` by ``ratio`` with negative
        parts:

        >>> abjad.mathtools.partition_integer_by_ratio(-10, [1, -2])
        [-3, 7]

    ..  container:: example

        More examples:

        >>> abjad.mathtools.partition_integer_by_ratio(10, [1])
        [10]

        >>> abjad.mathtools.partition_integer_by_ratio(10, [1, 1])
        [5, 5]

        >>> abjad.mathtools.partition_integer_by_ratio(10, [1, -1, -1])
        [3, -4, -3]

        >>> abjad.mathtools.partition_integer_by_ratio(-10, [1, 1, 1, 1])
        [-3, -2, -3, -2]

        >>> abjad.mathtools.partition_integer_by_ratio(-10, [1, 1, 1, 1, 1])
        [-2, -2, -2, -2, -2]

    Returns result with weight equal to absolute value of ``n``.

    Returns list of integers.
    """
    from abjad import mathtools
    if not mathtools.is_integer_equivalent_number(n):
        message = 'is not integer-equivalent number: {!r}.'
        message = message.format(n)
        raise TypeError(message)
    ratio = mathtools.Ratio(ratio).numbers
    if not all(
        mathtools.is_integer_equivalent_number(part)
        for part in ratio
        ):
        message = 'some parts in {!r} not integer-equivalent numbers.'
        message = message.format(ratio)
        raise TypeError(message)
    result = [0]
    divisions = [
        float(abs(n)) * abs(part) / mathtools.weight(ratio)
        for part in ratio
        ]
    cumulative_divisions = mathtools.cumulative_sums(divisions, start=None)
    for division in cumulative_divisions:
        rounded_division = int(round(division)) - sum(result)
        if division - round(division) == 0.5:
            rounded_division += 1
        result.append(rounded_division)
    result = result[1:]
    if mathtools.sign(n) == -1:
        result = [-x for x in result]
    ratio_signs = [mathtools.sign(x) for x in ratio]
    result = [pair[0] * pair[1] for pair in zip(ratio_signs, result)]
    return result



import abc
import bisect
import copy
import uqbar.graphs
from abjad import enums
from abjad import exceptions
from abjad import mathtools
from abjad.system.AbjadObject import AbjadObject
from abjad.system.FormatSpecification import FormatSpecification
from abjad.system.LilyPondFormatManager import LilyPondFormatManager
from abjad.system.StorageFormatManager import StorageFormatManager
from abjad.system.UpdateManager import UpdateManager
from abjad.system.Wrapper import Wrapper
from abjad.timespans.Timespan import Timespan
from abjad.utilities.Duration import Duration


class Component(AbjadObject):
    """
    Component baseclass.
    """

    ### CLASS VARIABLES ###

    __slots__ = (
        '_indicators_are_current',
        '_is_forbidden_to_update',
        '_overrides',
        '_lilypond_setting_name_manager',
        '_measure_number',
        '_offsets_are_current',
        '_offsets_in_seconds_are_current',
        '_parent',
        '_start_offset',
        '_start_offset_in_seconds',
        '_stop_offset',
        '_stop_offset_in_seconds',
        '_timespan',
        '_wrappers',
        )

    ### INITIALIZER ###

    @abc.abstractmethod
    def __init__(self, name=None):
        self._indicators_are_current = False
        self._is_forbidden_to_update = False
        self._measure_number = None
        self._offsets_are_current = False
        self._offsets_in_seconds_are_current = False
        self._overrides = None
        self._parent = None
        self._lilypond_setting_name_manager = None
        self._start_offset = None
        self._start_offset_in_seconds = None
        self._stop_offset = None
        self._stop_offset_in_seconds = None
        self._timespan = Timespan()
        self._wrappers = []

    ### SPECIAL METHODS ###

    def __copy__(self, *arguments):
        """
        Shallow copies component.

        Copies indicators.

        Does not copy spanners.

        Does not copy children.

        Returns new component.
        """
        import abjad
        new_component = type(self)(*self.__getnewargs__())
        if getattr(self, '_overrides', None) is not None:
            manager = copy.copy(abjad.override(self))
            new_component._overrides = manager
        if getattr(self, '_lilypond_setting_name_manager', None) is not None:
            manager = copy.copy(abjad.setting(self))
            new_component._lilypond_setting_name_manager = manager
        for wrapper in abjad.inspect(self).annotation_wrappers():
            new_wrapper = copy.copy(wrapper)
            abjad.attach(new_wrapper, new_component)
        for wrapper in abjad.inspect(self).wrappers():
            new_wrapper = copy.copy(wrapper)
            abjad.attach(new_wrapper, new_component)
        return new_component

    def __format__(self, format_specification=''):
        """
        Formats component.

        Returns string.
        """
        if format_specification in ('', 'lilypond'):
            return self._get_lilypond_format()
        elif format_specification == 'storage':
            return StorageFormatManager(self).get_storage_format()
        raise ValueError(repr(format_specification))

    def __getnewargs__(self):
        """
        Gets new arguments.

        Returns tuple.
        """
        return ()

    def __illustrate__(self):
        """
        Illustrates component.

        Returns LilyPond file.
        """
        import abjad
        lilypond_file = abjad.LilyPondFile.new(self)
        return lilypond_file

    def __mul__(self, n):
        """
        Copies component `n` times and detaches spanners.

        Returns list of new components.
        """
        import abjad
        components = []
        for i in range(n):
            component = abjad.mutate(self).copy()
            components.append(component)
        for component in abjad.iterate(components).components():
            abjad.detach(abjad.Spanner, component)
        result = abjad.select(components)
        return result

    def __repr__(self):
        """
        Gets interpreter representation of leaf.

        Returns string.
        """
        return StorageFormatManager(self).get_repr_format()

    def __rmul__(self, n):
        """
        Copies component `n` times and detach spanners.

        Returns list of new components.
        """
        return self * n

    ### PRIVATE METHODS ###

    def _as_graphviz_node(self):
        import abjad
        score_index = abjad.inspect(self).parentage().score_index
        score_index = '_'.join(str(_) for _ in score_index)
        class_name = type(self).__name__
        if score_index:
            name = '{}_{}'.format(class_name, score_index)
        else:
            name = class_name
        node = uqbar.graphs.Node(
            name=name,
            attributes={
                'margin': 0.05,
                'style': 'rounded',
                },
            )
        table = uqbar.graphs.Table(
            attributes={
                'border': 2,
                'cellpadding': 5,
                },
            )
        node.append(table)
        return node

    def _cache_named_children(self):
        name_dictionary = {}
        if hasattr(self, '_named_children'):
            for name, children in self._named_children.items():
                name_dictionary[name] = copy.copy(children)
        name = getattr(self, 'name', None)
        if name is not None:
            if self.name not in name_dictionary:
                name_dictionary[self.name] = []
            name_dictionary[self.name].append(self)
        return name_dictionary

    def _check_for_cycles(self, components):
        import abjad
        parentage = abjad.inspect(self).parentage()
        for component in components:
            if component in parentage:
                return True
        return False

    def _extract(self, scale_contents=False):
        import abjad
        if scale_contents:
            self._scale_contents(self.multiplier)
        selection = abjad.select([self])
        parent, start, stop = selection._get_parent_and_start_stop_indices()
        components = list(getattr(self, 'components', ()))
        parent.__setitem__(slice(start, stop + 1), components)
        return self

    def _format_absolute_after_slot(self, bundle):
        return []

    def _format_absolute_before_slot(self, bundle):
        return []

    def _format_after_slot(self, bundle):
        pass

    def _format_before_slot(self, bundle):
        pass

    def _format_close_brackets_slot(self, bundle):
        pass

    def _format_closing_slot(self, bundle):
        pass

    def _format_component(self, pieces=False):
        result = []
        bundle = LilyPondFormatManager.bundle_format_contributions(self)
        result.extend(self._format_absolute_before_slot(bundle))
        result.extend(self._format_before_slot(bundle))
        result.extend(self._format_open_brackets_slot(bundle))
        result.extend(self._format_opening_slot(bundle))
        result.extend(self._format_contents_slot(bundle))
        result.extend(self._format_closing_slot(bundle))
        result.extend(self._format_close_brackets_slot(bundle))
        result.extend(self._format_after_slot(bundle))
        result.extend(self._format_absolute_after_slot(bundle))
        contributions = []
        for contributor, contribution in result:
            contributions.extend(contribution)
        if pieces:
            return contributions
        else:
            return '\n'.join(contributions)

    def _format_contents_slot(self, bundle):
        pass

    def _format_open_brackets_slot(self, bundle):
        pass

    def _format_opening_slot(self, bundle):
        pass

    def _get_contents(self, include_self=True):
        import abjad
        result = []
        if include_self:
            result.append(self)
        result.extend(getattr(self, 'components', []))
        result = abjad.select(result)
        return result

    def _get_descendants(self, include_self=True):
        import abjad
        return abjad.Descendants(self, include_self=include_self)

    def _get_descendants_starting_with(self):
        import abjad
        result = []
        result.append(self)
        if isinstance(self, abjad.Container):
            if self.is_simultaneous:
                for x in self:
                    result.extend(x._get_descendants_starting_with())
            elif self:
                result.extend(self[0]._get_descendants_starting_with())
        return result

    def _get_descendants_stopping_with(self):
        import abjad
        result = []
        result.append(self)
        if isinstance(self, abjad.Container):
            if self.is_simultaneous:
                for x in self:
                    result.extend(x._get_descendants_stopping_with())
            elif self:
                result.extend(self[-1]._get_descendants_stopping_with())
        return result

    def _get_duration(self, in_seconds=False):
        import abjad
        if in_seconds:
            return self._get_duration_in_seconds()
        else:
            parentage = abjad.inspect(self).parentage(include_self=False)
            return parentage.prolation * self._get_preprolated_duration()

    def _get_effective(self, prototype, unwrap=True, n=0):
        import abjad
        # return time signature attached to measure regardless of context
        if (prototype == abjad.TimeSignature or
            prototype == (abjad.TimeSignature,)):
            if isinstance(self, abjad.Measure):
                if self._has_indicator(abjad.TimeSignature):
                    indicator = self._get_indicator(abjad.TimeSignature)
                    return indicator
                else:
                    return
        self._update_now(indicators=True)
        candidate_wrappers = {}
        parentage = abjad.inspect(self).parentage(
            include_self=True,
            grace_notes=True,
            )
        for component in parentage:
            these_wrappers = []
            for wrapper in component._wrappers:
                if wrapper.annotation:
                    continue
                if isinstance(wrapper.indicator, prototype):
                    #offset = wrapper.start_offset
                    #candidate_wrappers.setdefault(offset, []).append(wrapper)
                    these_wrappers.append(wrapper)
            # activate indicator takes precendence over inactive indicator
            if (any(_.deactivate is True for _ in these_wrappers) and
                not all(_.deactivate is True for _ in these_wrappers)):
                these_wrappers = [
                    _ for _ in these_wrappers if _.deactivate is not True
                    ]
            for wrapper in these_wrappers:
                offset = wrapper.start_offset
                candidate_wrappers.setdefault(offset, []).append(wrapper)
            if not isinstance(component, abjad.Context):
                continue
            for wrapper in component._dependent_wrappers:
                if wrapper.annotation:
                    continue
                if isinstance(wrapper.indicator, prototype):
                    offset = wrapper.start_offset
                    candidate_wrappers.setdefault(offset, []).append(wrapper)
        if not candidate_wrappers:
            return
        all_offsets = sorted(candidate_wrappers)
        start_offset = abjad.inspect(self).timespan().start_offset
        index = bisect.bisect(all_offsets, start_offset) - 1 + int(n)
        if index < 0:
            return
        elif len(candidate_wrappers) <= index:
            return
        wrapper = candidate_wrappers[all_offsets[index]][0]
        if unwrap:
            return wrapper.indicator
        return wrapper

    def _get_effective_staff(self):
        import abjad
        staff_change = self._get_effective(abjad.StaffChange)
        if staff_change is not None:
            effective_staff = staff_change.staff
        else:
            parentage = abjad.inspect(self).parentage()
            effective_staff = parentage.get_first(abjad.Staff)
        return effective_staff

    def _get_format_contributions_for_slot(self, slot_identifier, bundle=None):
        result = []
        if bundle is None:
            manager = LilyPondFormatManager
            bundle = manager.bundle_format_contributions(self)
        slot_names = (
            'before',
            'open_brackets',
            'opening',
            'contents',
            'closing',
            'close_brackets',
            'after',
            )
        if isinstance(slot_identifier, int):
            assert slot_identifier in range(1, 7 + 1)
            slot_index = slot_identifier - 1
            slot_name = slot_names[slot_index]
        elif isinstance(slot_identifier, str):
            slot_name = slot_identifier.replace(' ', '_')
            assert slot_name in slot_names
        method_name = '_format_{}_slot'.format(slot_name)
        method = getattr(self, method_name)
        for source, contributions in method(bundle):
            result.extend(contributions)
        return result

    def _get_format_pieces(self):
        return self._format_component(pieces=True)

    def _get_format_specification(self):
        values = []
        summary = self._get_contents_summary()
        if summary:
            values.append(summary)
        return FormatSpecification(
            client=self,
            repr_args_values=values,
            storage_format_kwargs_names=[]
            )

    def _get_in_my_logical_voice(self, n, prototype=None):
        import abjad
        if 0 <= n:
            generator = abjad.iterate(self)._logical_voice(
                prototype=prototype,
                reverse=False,
                )
            for i, component in enumerate(generator):
                if i == n:
                    return component
        else:
            n = abs(n)
            generator = abjad.iterate(self)._logical_voice(
                prototype=prototype,
                reverse=True,
                )
            for i, component in enumerate(generator):
                if i == n:
                    return component

    def _get_indicator(self, prototype=None, unwrap=True):
        indicators = self._get_indicators(prototype=prototype, unwrap=unwrap)
        if not indicators:
            message = 'no attached indicators found matching {!r}.'
            message = message.format(prototype)
            raise ValueError(message)
        if 1 < len(indicators):
            message = 'multiple attached indicators found matching {!r}.'
            message = message.format(prototype)
            raise ValueError(message)
        return indicators[0]

    def _get_indicators(self, prototype=None, unwrap=True):
        prototype = prototype or (object,)
        if not isinstance(prototype, tuple):
            prototype = (prototype,)
        prototype_objects, prototype_classes = [], []
        for indicator_prototype in prototype:
            if isinstance(indicator_prototype, type):
                prototype_classes.append(indicator_prototype)
            else:
                prototype_objects.append(indicator_prototype)
        prototype_objects = tuple(prototype_objects)
        prototype_classes = tuple(prototype_classes)
        result = []
        for wrapper in self._wrappers:
            if wrapper.annotation:
                continue
            if isinstance(wrapper, prototype_classes):
                result.append(wrapper)
            elif any(wrapper == _ for _ in prototype_objects):
                result.append(wrapper)
            elif isinstance(wrapper, Wrapper):
                if isinstance(wrapper.indicator, prototype_classes):
                    result.append(wrapper)
                elif any(wrapper.indicator == _ for _ in prototype_objects):
                    result.append(wrapper)
        if unwrap:
            result = [_.indicator for _ in result]
        result = tuple(result)
        return result

    def _get_lilypond_format(self):
        self._update_now(indicators=True)
        return self._format_component()

    def _get_lineage(self):
        import abjad
        return abjad.Lineage(self)

    def _get_markup(self, direction=None):
        import abjad
        markup = self._get_indicators(abjad.Markup)
        if direction is enums.Up:
            return tuple(x for x in markup if x.direction is enums.Up)
        elif direction is enums.Down:
            return tuple(x for x in markup if x.direction is enums.Down)
        return markup

    def _get_next_measure(self):
        import abjad
        if isinstance(self, abjad.Leaf):
            for parent in abjad.inspect(self).parentage(
                include_self=False):
                if isinstance(parent, abjad.Measure):
                    return parent
            raise exceptions.MissingMeasureError
        elif isinstance(self, abjad.Measure):
            return self._get_in_my_logical_voice(1, prototype=abjad.Measure)
        elif isinstance(self, abjad.Container):
            contents = self._get_descendants_starting_with()
            contents = [x for x in contents if isinstance(x, abjad.Measure)]
            if contents:
                return contents[0]
            raise exceptions.MissingMeasureError
        elif isinstance(self, (list, tuple)):
            measure_generator = abjad.iterate(self).components(abjad.Measure)
            try:
                measure = next(measure_generator)
                return measure
            except StopIteration:
                raise exceptions.MissingMeasureError
        else:
            message = 'unknown component: {!r}.'
            raise TypeError(message.format(self))

    def _get_nth_component_in_time_order_from(self, n):
        import abjad
        assert mathtools.is_integer_equivalent(n)
        def next(component):
            if component is not None:
                for parent in abjad.inspect(component).parentage(
                    include_self=True):
                    next_sibling = parent._get_sibling(1)
                    if next_sibling is not None:
                        return next_sibling
        def previous(component):
            if component is not None:
                for parent in abjad.inspect(component).parentage(
                    include_self=True):
                    next_sibling = parent._get_sibling(-1)
                    if next_sibling is not None:
                        return next_sibling
        result = self
        if 0 < n:
            for i in range(n):
                result = next(result)
        elif n < 0:
            for i in range(abs(n)):
                result = previous(result)
        return result

    def _get_parentage(self, include_self=True, grace_notes=False):
        import abjad
        return abjad.Parentage(
            self,
            include_self=include_self,
            grace_notes=grace_notes,
            )

    def _get_previous_measure(self):
        import abjad
        if isinstance(self, abjad.Leaf):
            for parent in abjad.inspect(self).parentage(
                include_self=False):
                if isinstance(parent, abjad.Measure):
                    return parent
            raise exceptions.MissingMeasureError
        elif isinstance(self, abjad.Measure):
            return self._get_in_my_logical_voice(-1, prototype=abjad.Measure)
        elif isinstance(self, abjad.Container):
            contents = self._get_descendants_stopping_with()
            contents = [x for x in contents if isinstance(x, abjad.Measure)]
            if contents:
                return contents[0]
            raise exceptions.MissingMeasureError
        elif isinstance(self, (list, tuple)):
            measure_generator = abjad.iterate(self).components(
                abjad.Measure,
                reverse=True,
                )
            try:
                measure = next(measure_generator)
                return measure
            except StopIteration:
                raise exceptions.MissingMeasureError
        else:
            message = 'unknown component: {!r}.'
            raise TypeError(message.format(self))

    def _get_sibling(self, n):
        if n == 0:
            return self
        elif 0 < n:
            if self._parent is not None:
                if not self._parent.is_simultaneous:
                    index = self._parent.index(self)
                    if index + n < len(self._parent):
                        return self._parent[index + n]
        elif n < 0:
            if self._parent is not None:
                if not self._parent.is_simultaneous:
                    index = self._parent.index(self)
                    if 0 <= index + n:
                        return self._parent[index + n]

    def _get_timespan(self, in_seconds=False):
        import abjad
        if in_seconds:
            self._update_now(offsets_in_seconds=True)
            if self._start_offset_in_seconds is None:
                raise exceptions.MissingMetronomeMarkError
            return Timespan(
                start_offset=self._start_offset_in_seconds,
                stop_offset=self._stop_offset_in_seconds,
                )
        else:
            self._update_now(offsets=True)
            return self._timespan

    def _get_vertical_moment(self, governor=None):
        import abjad
        offset = abjad.inspect(self).timespan().start_offset
        if governor is None:
            governor = abjad.inspect(self).parentage().root
        return abjad.VerticalMoment(governor, offset)

    def _get_vertical_moment_at(self, offset):
        import abjad
        return abjad.VerticalMoment(self, offset)

    def _has_effective_indicator(self, prototype=None):
        indicator = self._get_effective(prototype=prototype)
        return indicator is not None

    def _has_indicator(self, prototype=None):
        indicators = self._get_indicators(prototype=prototype)
        return bool(indicators)

    def _is_immediate_temporal_successor_of(self, component):
        temporal_successors = []
        current = self
        while current is not None:
            next_sibling = current._get_sibling(1)
            if next_sibling is None:
                current = current._parent
            else:
                descendants = next_sibling._get_descendants_starting_with()
                temporal_successors = descendants
                break
        return component in temporal_successors

    def _move_indicators(self, recipient_component):
        import abjad
        for wrapper in abjad.inspect(self).wrappers():
            abjad.detach(wrapper, self)
            abjad.attach(wrapper, recipient_component)

    # TODO: eventually reimplement as a keyword option to remove()
    def _remove_and_shrink_durated_parent_containers(self):
        import abjad
        prolated_leaf_duration = self._get_duration()
        parentage = abjad.inspect(self).parentage(include_self=False)
        prolations = parentage._prolations
        current_prolation, i = Duration(1), 0
        parent = self._parent
        while parent is not None and not parent.is_simultaneous:
            current_prolation *= prolations[i]
            if isinstance(parent, abjad.Measure):
                indicator = parent._get_indicator(abjad.TimeSignature)
                parent_time_signature = indicator
                old_prolation = parent_time_signature.implied_prolation
                naive_time_signature = (
                    parent_time_signature.duration - prolated_leaf_duration)
                better_time_signature = abjad.NonreducedFraction(
                    naive_time_signature)
                better_time_signature = better_time_signature.with_denominator(
                    parent_time_signature.denominator)
                better_time_signature = abjad.TimeSignature(
                    better_time_signature)
                abjad.detach(abjad.TimeSignature, parent)
                abjad.attach(better_time_signature, parent)
                indicator = parent._get_indicator(abjad.TimeSignature)
                parent_time_signature = indicator
                new_prolation = parent_time_signature.implied_prolation
                adjusted_prolation = old_prolation / new_prolation
                for x in parent:
                    if adjusted_prolation != 1:
                        new_target = x._get_preprolated_duration()
                        new_target *= adjusted_prolation
                        contents_duration = abjad.inspect(x)
                        multiplier = new_target / contents_duration
                        tuplet = abjad.Tuplet(multiplier, [])
                        abjad.mutate(x).wrap(tuplet)
            parent = parent._parent
            i += 1
        parentage = abjad.inspect(self).parentage(include_self=False)
        parent = self._parent
        if parent:
            index = parent.index(self)
            del(parent[index])
        for x in parentage:
            if not len(x):
                x._extract()
            else:
                break

    def _remove_from_parent(self):
        import abjad
        self._update_later(offsets=True)
        for component in abjad.inspect(self).parentage(include_self=False):
            if not isinstance(component, abjad.Context):
                continue
            for wrapper in component._dependent_wrappers[:]:
                if wrapper.component is self:
                    component._dependent_wrappers.remove(wrapper)
        if self._parent is not None:
            self._parent._components.remove(self)
        self._parent = None

    def _remove_named_children_from_parentage(self, name_dictionary):
        import abjad
        if self._parent is not None and name_dictionary:
            for parent in abjad.inspect(self).parentage(
                include_self=False):
                named_children = parent._named_children
                for name in name_dictionary:
                    for component in name_dictionary[name]:
                        named_children[name].remove(component)
                    if not named_children[name]:
                        del named_children[name]

    def _restore_named_children_to_parentage(self, name_dictionary):
        import abjad
        if self._parent is not None and name_dictionary:
            for parent in abjad.inspect(self).parentage(
                include_self=False):
                named_children = parent._named_children
                for name in name_dictionary:
                    if name in named_children:
                        named_children[name].extend(name_dictionary[name])
                    else:
                        named_children[name] = copy.copy(name_dictionary[name])

    def _set_parent(self, new_parent):
        """
        Not composer-safe.
        """
        named_children = self._cache_named_children()
        self._remove_named_children_from_parentage(named_children)
        self._remove_from_parent()
        self._parent = new_parent
        self._restore_named_children_to_parentage(named_children)
        self._update_later(offsets=True)

    def _splice(
        self,
        components,
        direction=enums.Right,
        grow_spanners=True,
        ):
        import abjad
        assert all(isinstance(x, Component) for x in components)
        selection = abjad.select(self)
        if direction is enums.Right:
            if grow_spanners:
                insert_offset = abjad.inspect(self).timespan().stop_offset
                receipt = selection._get_dominant_spanners()
                for spanner, index in receipt:
                    insert_component = None
                    for component in spanner:
                        start_offset = abjad.inspect(
                            component).timespan().start_offset
                        if start_offset == insert_offset:
                            insert_component = component
                            break
                    if insert_component is not None:
                        insert_index = spanner._index(insert_component)
                    else:
                        insert_index = len(spanner)
                    for component in reversed(components):
                        leaves = abjad.select(component).leaves()
                        for leaf in reversed(leaves):
                            spanner._insert(insert_index, leaf)
                            leaf._append_spanner(spanner)
            selection = abjad.select(self)
            parent, start, stop = \
                selection._get_parent_and_start_stop_indices()
            if parent is not None:
                if grow_spanners:
                    for component in reversed(components):
                        component._set_parent(parent)
                        parent._components.insert(start + 1, component)
                else:
                    after = stop + 1
                    parent.__setitem__(slice(after, after), components)
            return [self] + components
        else:
            if grow_spanners:
                offset = abjad.inspect(self).timespan().start_offset
                receipt = selection._get_dominant_spanners()
                for spanner, x in receipt:
                    for component in spanner:
                        timespan = abjad.inspect(component).timespan()
                        if timespan.start_offset == offset:
                            index = spanner._index(component)
                            break
                    else:
                        raise ValueError('no component in spanner at offset.')
                    for component in reversed(components):
                        leaves = abjad.select(component).leaves()
                        for leaf in reversed(leaves):
                            spanner._insert(index, leaf)
                            if isinstance(component, abjad.Leaf):
                                component._append_spanner(spanner)
            selection = abjad.select(self)
            parent, start, stop = \
                selection._get_parent_and_start_stop_indices()
            if parent is not None:
                if grow_spanners:
                    for component in reversed(components):
                        component._set_parent(parent)
                        parent._components.insert(start, component)
                else:
                    parent.__setitem__(slice(start, start), components)
            return components + [self]

    def _update_later(self, offsets=False, offsets_in_seconds=False):
        import abjad
        assert offsets or offsets_in_seconds
        for component in abjad.inspect(self).parentage(include_self=True):
            if offsets:
                component._offsets_are_current = False
            elif offsets_in_seconds:
                component._offsets_in_seconds_are_current = False

    def _update_measure_numbers(self):
        update_manager = UpdateManager()
        update_manager._update_measure_numbers(self)

    def _update_now(
        self,
        offsets=False,
        offsets_in_seconds=False,
        indicators=False,
        ):
        update_manager = UpdateManager()
        return update_manager._update_now(
            self,
            offsets=offsets,
            offsets_in_seconds=offsets_in_seconds,
            indicators=indicators,
            )



from abjad.utilities.OrderedDict import OrderedDict
from .ScoreTemplate import ScoreTemplate


class GroupedStavesScoreTemplate(ScoreTemplate):
    r"""
    Grouped staves score template.

    ..  container:: example

        >>> class_ = abjad.GroupedStavesScoreTemplate
        >>> template = class_(staff_count=4)
        >>> abjad.show(template) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(template.__illustrate__()[abjad.Score])
            \context Score = "Grouped Staves Score"
            <<
                \context StaffGroup = "Grouped Staves Staff Group"
                <<
                    \context Staff = "Staff 1"
                    {
                        \context Voice = "Voice 1"
                        {
                            s1
                        }
                    }
                    \context Staff = "Staff 2"
                    {
                        \context Voice = "Voice 2"
                        {
                            s1
                        }
                    }
                    \context Staff = "Staff 3"
                    {
                        \context Voice = "Voice 3"
                        {
                            s1
                        }
                    }
                    \context Staff = "Staff 4"
                    {
                        \context Voice = "Voice 4"
                        {
                            s1
                        }
                    }
                >>
            >>

        >>> score = template()
        >>> abjad.f(score)
        \context Score = "Grouped Staves Score"
        <<
            \context StaffGroup = "Grouped Staves Staff Group"
            <<
                \context Staff = "Staff 1"
                {
                    \context Voice = "Voice 1"
                    {
                    }
                }
                \context Staff = "Staff 2"
                {
                    \context Voice = "Voice 2"
                    {
                    }
                }
                \context Staff = "Staff 3"
                {
                    \context Voice = "Voice 3"
                    {
                    }
                }
                \context Staff = "Staff 4"
                {
                    \context Voice = "Voice 4"
                    {
                    }
                }
            >>
        >>

    """

    ### CLASS VARIABLES ###

    __slots__ = (
        '_staff_count',
        )

    ### INITIALIZER ###

    def __init__(self, staff_count=2):
        super().__init__()
        self._staff_count = staff_count

    ### SPECIAL METHODS ###

    def __call__(self):
        """
        Calls score template.

        Returns score.
        """
        import abjad
        staves = []
        for index in range(self.staff_count):
            number = index + 1
            voice = abjad.Voice(
                [],
                name='Voice {}'.format(number),
                )
            staff = abjad.Staff(
                [voice],
                name='Staff {}'.format(number),
                )
            staves.append(staff)
            self.voice_abbreviations['v{}'.format(number)] = voice.name
        staff_group = abjad.StaffGroup(
            staves,
            name='Grouped Staves Staff Group',
            )
        score = abjad.Score(
            [staff_group],
            name='Grouped Staves Score',
            )
        return score

    ### PUBLIC PROPERTIES ###

    @property
    def voice_abbreviations(self):
        """
        Gets context name abbreviations.

        ..  container::

            >>> class_ = abjad.GroupedStavesScoreTemplate
            >>> template = class_(staff_count=4)
            >>> template.voice_abbreviations
            OrderedDict([])

        """
        return super().voice_abbreviations

    @property
    def staff_count(self):
        """
        Gets staff count.

        ..  container::

            >>> class_ = abjad.GroupedStavesScoreTemplate
            >>> template = class_(staff_count=4)
            >>> template.staff_count
            4

        """
        return self._staff_count



def integer_to_binary_string(n):
    r"""
    Changes positive integer ``n`` to binary string.

    ..  container:: example

        >>> for n in range(1, 16 + 1):
        ...     print('{}\t{}'.format(n, abjad.mathtools.integer_to_binary_string(n)))
        ...
        1  1
        2  10
        3  11
        4  100
        5  101
        6  110
        7  111
        8  1000
        9  1001
        10 1010
        11 1011
        12 1100
        13 1101
        14 1110
        15 1111
        16 10000

    Returns string.
    """
    if n == 0:
        return '0'
    result = bin(abs(n)).lstrip('-0b')
    if n < 0:
        result = '-' + result
    return result



lilypond_version = "2.19.24"

grob_interfaces = {
    "Accidental": [
        "accidental-interface",
        "font-interface",
        "grob-interface",
        "inline-accidental-interface",
        "item-interface",
        ],
    "AccidentalCautionary": [
        "accidental-interface",
        "font-interface",
        "grob-interface",
        "inline-accidental-interface",
        "item-interface",
        ],
    "AccidentalPlacement": [
        "accidental-placement-interface",
        "grob-interface",
        "item-interface",
        ],
    "AccidentalSuggestion": [
        "accidental-interface",
        "accidental-suggestion-interface",
        "font-interface",
        "grob-interface",
        "item-interface",
        "outside-staff-interface",
        "script-interface",
        "self-alignment-interface",
        "side-position-interface",
        ],
    "Ambitus": [
        "ambitus-interface",
        "axis-group-interface",
        "break-aligned-interface",
        "grob-interface",
        "item-interface",
        ],
    "AmbitusAccidental": [
        "accidental-interface",
        "break-aligned-interface",
        "font-interface",
        "grob-interface",
        "item-interface",
        "side-position-interface",
        ],
    "AmbitusLine": [
        "ambitus-interface",
        "font-interface",
        "grob-interface",
        "item-interface",
        ],
    "AmbitusNoteHead": [
        "ambitus-interface",
        "font-interface",
        "grob-interface",
        "item-interface",
        "ledgered-interface",
        "note-head-interface",
        "rhythmic-head-interface",
        "staff-symbol-referencer-interface",
        ],
    "Arpeggio": [
        "arpeggio-interface",
        "font-interface",
        "grob-interface",
        "item-interface",
        "side-position-interface",
        "staff-symbol-referencer-interface",
        ],
    "BalloonTextItem": [
        "balloon-interface",
        "font-interface",
        "grob-interface",
        "item-interface",
        "text-interface",
        ],
    "BarLine": [
        "bar-line-interface",
        "break-aligned-interface",
        "font-interface",
        "grob-interface",
        "item-interface",
        "pure-from-neighbor-interface",
        ],
    "BarNumber": [
        "break-alignable-interface",
        "font-interface",
        "grob-interface",
        "item-interface",
        "outside-staff-interface",
        "self-alignment-interface",
        "side-position-interface",
        "text-interface",
        ],
    "BassFigure": [
        "bass-figure-interface",
        "font-interface",
        "grob-interface",
        "item-interface",
        "rhythmic-grob-interface",
        "text-interface",
        ],
    "BassFigureAlignment": [
        "align-interface",
        "axis-group-interface",
        "bass-figure-alignment-interface",
        "grob-interface",
        "spanner-interface",
        ],
    "BassFigureAlignmentPositioning": [
        "axis-group-interface",
        "grob-interface",
        "outside-staff-interface",
        "side-position-interface",
        "spanner-interface",
        ],
    "BassFigureBracket": [
        "enclosing-bracket-interface",
        "grob-interface",
        "item-interface",
        ],
    "BassFigureContinuation": [
        "figured-bass-continuation-interface",
        "grob-interface",
        "spanner-interface",
        ],
    "BassFigureLine": [
        "axis-group-interface",
        "grob-interface",
        "outside-staff-axis-group-interface",
        "spanner-interface",
        ],
    "Beam": [
        "beam-interface",
        "font-interface",
        "grob-interface",
        "spanner-interface",
        "staff-symbol-referencer-interface",
        "unbreakable-spanner-interface",
        ],
    "BendAfter": [
        "bend-after-interface",
        "grob-interface",
        "spanner-interface",
        ],
    "BreakAlignGroup": [
        "axis-group-interface",
        "break-aligned-interface",
        "grob-interface",
        "item-interface",
        ],
    "BreakAlignment": [
        "axis-group-interface",
        "break-alignment-interface",
        "grob-interface",
        "item-interface",
        ],
    "BreathingSign": [
        "break-aligned-interface",
        "breathing-sign-interface",
        "font-interface",
        "grob-interface",
        "item-interface",
        "outside-staff-interface",
        "text-interface",
        ],
    "ChordName": [
        "chord-name-interface",
        "font-interface",
        "grob-interface",
        "item-interface",
        "outside-staff-interface",
        "rhythmic-grob-interface",
        "text-interface",
        ],
    "Clef": [
        "break-aligned-interface",
        "clef-interface",
        "font-interface",
        "grob-interface",
        "item-interface",
        "pure-from-neighbor-interface",
        "staff-symbol-referencer-interface",
        ],
    "ClefModifier": [
        "clef-modifier-interface",
        "font-interface",
        "grob-interface",
        "item-interface",
        "outside-staff-interface",
        "self-alignment-interface",
        "side-position-interface",
        "text-interface",
        ],
    "ClusterSpanner": [
        "cluster-interface",
        "grob-interface",
        "spanner-interface",
        ],
    "ClusterSpannerBeacon": [
        "cluster-beacon-interface",
        "grob-interface",
        "item-interface",
        "rhythmic-grob-interface",
        ],
    "CombineTextScript": [
        "font-interface",
        "grob-interface",
        "item-interface",
        "outside-staff-interface",
        "self-alignment-interface",
        "side-position-interface",
        "text-interface",
        "text-script-interface",
        ],
    "CueClef": [
        "break-aligned-interface",
        "clef-interface",
        "font-interface",
        "grob-interface",
        "item-interface",
        "pure-from-neighbor-interface",
        "staff-symbol-referencer-interface",
        ],
    "CueEndClef": [
        "break-aligned-interface",
        "clef-interface",
        "font-interface",
        "grob-interface",
        "item-interface",
        "pure-from-neighbor-interface",
        "staff-symbol-referencer-interface",
        ],
    "Custos": [
        "break-aligned-interface",
        "custos-interface",
        "font-interface",
        "grob-interface",
        "item-interface",
        "staff-symbol-referencer-interface",
        ],
    "DotColumn": [
        "axis-group-interface",
        "dot-column-interface",
        "grob-interface",
        "item-interface",
        ],
    "Dots": [
        "dots-interface",
        "font-interface",
        "grob-interface",
        "item-interface",
        "staff-symbol-referencer-interface",
        ],
    "DoublePercentRepeat": [
        "break-aligned-interface",
        "font-interface",
        "grob-interface",
        "item-interface",
        "percent-repeat-interface",
        "percent-repeat-item-interface",
        ],
    "DoublePercentRepeatCounter": [
        "font-interface",
        "grob-interface",
        "item-interface",
        "outside-staff-interface",
        "percent-repeat-interface",
        "percent-repeat-item-interface",
        "self-alignment-interface",
        "side-position-interface",
        "text-interface",
        ],
    "DoubleRepeatSlash": [
        "font-interface",
        "grob-interface",
        "item-interface",
        "outside-staff-interface",
        "percent-repeat-interface",
        "percent-repeat-item-interface",
        "rhythmic-grob-interface",
        ],
    "DynamicLineSpanner": [
        "axis-group-interface",
        "dynamic-interface",
        "dynamic-line-spanner-interface",
        "grob-interface",
        "outside-staff-interface",
        "side-position-interface",
        "spanner-interface",
        ],
    "DynamicText": [
        "dynamic-interface",
        "dynamic-text-interface",
        "font-interface",
        "grob-interface",
        "item-interface",
        "outside-staff-interface",
        "script-interface",
        "self-alignment-interface",
        "text-interface",
        ],
    "DynamicTextSpanner": [
        "dynamic-interface",
        "dynamic-text-spanner-interface",
        "font-interface",
        "grob-interface",
        "line-interface",
        "line-spanner-interface",
        "spanner-interface",
        "text-interface",
        ],
    "Episema": [
        "episema-interface",
        "font-interface",
        "grob-interface",
        "line-interface",
        "line-spanner-interface",
        "side-position-interface",
        "spanner-interface",
        ],
    "Fingering": [
        "finger-interface",
        "font-interface",
        "grob-interface",
        "item-interface",
        "outside-staff-interface",
        "self-alignment-interface",
        "side-position-interface",
        "text-interface",
        "text-script-interface",
        ],
    "FingeringColumn": [
        "fingering-column-interface",
        "grob-interface",
        "item-interface",
        ],
    "Flag": [
        "flag-interface",
        "font-interface",
        "grob-interface",
        "item-interface",
        ],
    "FootnoteItem": [
        "balloon-interface",
        "font-interface",
        "footnote-interface",
        "grob-interface",
        "item-interface",
        "text-interface",
        ],
    "FootnoteSpanner": [
        "balloon-interface",
        "font-interface",
        "footnote-interface",
        "footnote-spanner-interface",
        "grob-interface",
        "spanner-interface",
        "text-interface",
        ],
    "FretBoard": [
        "chord-name-interface",
        "font-interface",
        "fret-diagram-interface",
        "grob-interface",
        "item-interface",
        "outside-staff-interface",
        "rhythmic-grob-interface",
        ],
    "Glissando": [
        "glissando-interface",
        "grob-interface",
        "line-interface",
        "line-spanner-interface",
        "spanner-interface",
        "unbreakable-spanner-interface",
        ],
    "GraceSpacing": [
        "grace-spacing-interface",
        "grob-interface",
        "spacing-options-interface",
        "spanner-interface",
        ],
    "GridLine": [
        "grid-line-interface",
        "grob-interface",
        "item-interface",
        "self-alignment-interface",
        ],
    "GridPoint": [
        "grid-point-interface",
        "grob-interface",
        "item-interface",
        ],
    "Hairpin": [
        "dynamic-interface",
        "grob-interface",
        "hairpin-interface",
        "line-interface",
        "outside-staff-interface",
        "self-alignment-interface",
        "spanner-interface",
        ],
    "HorizontalBracket": [
        "grob-interface",
        "horizontal-bracket-interface",
        "line-interface",
        "outside-staff-interface",
        "side-position-interface",
        "spanner-interface",
        ],
    "InstrumentName": [
        "font-interface",
        "grob-interface",
        "self-alignment-interface",
        "side-position-interface",
        "spanner-interface",
        "system-start-text-interface",
        "text-interface",
        ],
    "InstrumentSwitch": [
        "font-interface",
        "grob-interface",
        "item-interface",
        "outside-staff-interface",
        "self-alignment-interface",
        "side-position-interface",
        "text-interface",
        ],
    "KeyCancellation": [
        "break-aligned-interface",
        "font-interface",
        "grob-interface",
        "item-interface",
        "key-cancellation-interface",
        "key-signature-interface",
        "pure-from-neighbor-interface",
        "staff-symbol-referencer-interface",
        ],
    "KeySignature": [
        "break-aligned-interface",
        "font-interface",
        "grob-interface",
        "item-interface",
        "key-signature-interface",
        "pure-from-neighbor-interface",
        "staff-symbol-referencer-interface",
        ],
    "KievanLigature": [
        "font-interface",
        "grob-interface",
        "kievan-ligature-interface",
        "spanner-interface",
        ],
    "LaissezVibrerTie": [
        "grob-interface",
        "item-interface",
        "semi-tie-interface",
        ],
    "LaissezVibrerTieColumn": [
        "grob-interface",
        "item-interface",
        "semi-tie-column-interface",
        ],
    "LedgerLineSpanner": [
        "grob-interface",
        "ledger-line-spanner-interface",
        "spanner-interface",
        ],
    "LeftEdge": [
        "break-aligned-interface",
        "grob-interface",
        "item-interface",
        ],
    "LigatureBracket": [
        "grob-interface",
        "line-interface",
        "spanner-interface",
        "tuplet-bracket-interface",
        ],
    "LyricExtender": [
        "grob-interface",
        "lyric-extender-interface",
        "lyric-interface",
        "spanner-interface",
        ],
    "LyricHyphen": [
        "font-interface",
        "grob-interface",
        "lyric-hyphen-interface",
        "lyric-interface",
        "spanner-interface",
        ],
    "LyricSpace": [
        "grob-interface",
        "lyric-hyphen-interface",
        "spanner-interface",
        ],
    "LyricText": [
        "font-interface",
        "grob-interface",
        "item-interface",
        "lyric-syllable-interface",
        "rhythmic-grob-interface",
        "self-alignment-interface",
        "text-interface",
        ],
    "MeasureCounter": [
        "font-interface",
        "grob-interface",
        "measure-counter-interface",
        "outside-staff-interface",
        "self-alignment-interface",
        "side-position-interface",
        "spanner-interface",
        "text-interface",
        ],
    "MeasureGrouping": [
        "grob-interface",
        "measure-grouping-interface",
        "outside-staff-interface",
        "side-position-interface",
        "spanner-interface",
        ],
    "MelodyItem": [
        "grob-interface",
        "item-interface",
        "melody-spanner-interface",
        ],
    "MensuralLigature": [
        "font-interface",
        "grob-interface",
        "mensural-ligature-interface",
        "spanner-interface",
        ],
    "MetronomeMark": [
        "break-alignable-interface",
        "font-interface",
        "grob-interface",
        "item-interface",
        "metronome-mark-interface",
        "outside-staff-interface",
        "self-alignment-interface",
        "side-position-interface",
        "text-interface",
        ],
    "MultiMeasureRest": [
        "font-interface",
        "grob-interface",
        "multi-measure-interface",
        "multi-measure-rest-interface",
        "outside-staff-interface",
        "rest-interface",
        "spanner-interface",
        "staff-symbol-referencer-interface",
        ],
    "MultiMeasureRestNumber": [
        "font-interface",
        "grob-interface",
        "multi-measure-interface",
        "outside-staff-interface",
        "self-alignment-interface",
        "side-position-interface",
        "spanner-interface",
        "text-interface",
        ],
    "MultiMeasureRestText": [
        "font-interface",
        "grob-interface",
        "multi-measure-interface",
        "outside-staff-interface",
        "self-alignment-interface",
        "side-position-interface",
        "spanner-interface",
        "text-interface",
        ],
    "NonMusicalPaperColumn": [
        "axis-group-interface",
        "font-interface",
        "grob-interface",
        "item-interface",
        "paper-column-interface",
        "separation-item-interface",
        "spaceable-grob-interface",
        ],
    "NoteCollision": [
        "axis-group-interface",
        "grob-interface",
        "item-interface",
        "note-collision-interface",
        ],
    "NoteColumn": [
        "axis-group-interface",
        "grob-interface",
        "item-interface",
        "note-column-interface",
        "separation-item-interface",
        ],
    "NoteHead": [
        "font-interface",
        "gregorian-ligature-interface",
        "grob-interface",
        "item-interface",
        "ledgered-interface",
        "ligature-head-interface",
        "mensural-ligature-interface",
        "note-head-interface",
        "rhythmic-grob-interface",
        "rhythmic-head-interface",
        "staff-symbol-referencer-interface",
        "vaticana-ligature-interface",
        ],
    "NoteName": [
        "font-interface",
        "grob-interface",
        "item-interface",
        "note-name-interface",
        "text-interface",
        ],
    "NoteSpacing": [
        "grob-interface",
        "item-interface",
        "note-spacing-interface",
        "spacing-interface",
        ],
    "OttavaBracket": [
        "font-interface",
        "grob-interface",
        "horizontal-bracket-interface",
        "line-interface",
        "ottava-bracket-interface",
        "outside-staff-interface",
        "side-position-interface",
        "spanner-interface",
        "text-interface",
        ],
    "PaperColumn": [
        "axis-group-interface",
        "font-interface",
        "grob-interface",
        "item-interface",
        "paper-column-interface",
        "separation-item-interface",
        "spaceable-grob-interface",
        ],
    "ParenthesesItem": [
        "font-interface",
        "grob-interface",
        "item-interface",
        "parentheses-interface",
        ],
    "PercentRepeat": [
        "font-interface",
        "grob-interface",
        "multi-measure-rest-interface",
        "percent-repeat-interface",
        "spanner-interface",
        ],
    "PercentRepeatCounter": [
        "font-interface",
        "grob-interface",
        "outside-staff-interface",
        "percent-repeat-interface",
        "self-alignment-interface",
        "side-position-interface",
        "spanner-interface",
        "text-interface",
        ],
    "PhrasingSlur": [
        "grob-interface",
        "outside-staff-interface",
        "slur-interface",
        "spanner-interface",
        ],
    "PianoPedalBracket": [
        "grob-interface",
        "line-interface",
        "piano-pedal-bracket-interface",
        "piano-pedal-interface",
        "spanner-interface",
        ],
    "RehearsalMark": [
        "break-alignable-interface",
        "font-interface",
        "grob-interface",
        "item-interface",
        "mark-interface",
        "outside-staff-interface",
        "self-alignment-interface",
        "side-position-interface",
        "text-interface",
        ],
    "RepeatSlash": [
        "grob-interface",
        "item-interface",
        "percent-repeat-interface",
        "percent-repeat-item-interface",
        "rhythmic-grob-interface",
        ],
    "RepeatTie": [
        "grob-interface",
        "item-interface",
        "semi-tie-interface",
        ],
    "RepeatTieColumn": [
        "grob-interface",
        "item-interface",
        "semi-tie-column-interface",
        ],
    "Rest": [
        "font-interface",
        "grob-interface",
        "item-interface",
        "rest-interface",
        "rhythmic-grob-interface",
        "rhythmic-head-interface",
        "staff-symbol-referencer-interface",
        ],
    "RestCollision": [
        "grob-interface",
        "item-interface",
        "rest-collision-interface",
        ],
    "Script": [
        "font-interface",
        "grob-interface",
        "item-interface",
        "outside-staff-interface",
        "script-interface",
        "self-alignment-interface",
        "side-position-interface",
        ],
    "ScriptColumn": [
        "grob-interface",
        "item-interface",
        "script-column-interface",
        ],
    "ScriptRow": [
        "grob-interface",
        "item-interface",
        "script-column-interface",
        ],
    "Slur": [
        "grob-interface",
        "outside-staff-interface",
        "slur-interface",
        "spanner-interface",
        ],
    "SostenutoPedal": [
        "font-interface",
        "grob-interface",
        "item-interface",
        "piano-pedal-script-interface",
        "self-alignment-interface",
        "text-interface",
        ],
    "SostenutoPedalLineSpanner": [
        "axis-group-interface",
        "grob-interface",
        "outside-staff-interface",
        "piano-pedal-interface",
        "side-position-interface",
        "spanner-interface",
        ],
    "SpacingSpanner": [
        "grob-interface",
        "spacing-options-interface",
        "spacing-spanner-interface",
        "spanner-interface",
        ],
    "SpanBar": [
        "bar-line-interface",
        "font-interface",
        "grob-interface",
        "item-interface",
        "span-bar-interface",
        ],
    "SpanBarStub": [
        "grob-interface",
        "item-interface",
        "pure-from-neighbor-interface",
        ],
    "StaffGrouper": [
        "grob-interface",
        "spanner-interface",
        "staff-grouper-interface",
        ],
    "StaffSpacing": [
        "grob-interface",
        "item-interface",
        "spacing-interface",
        "staff-spacing-interface",
        ],
    "StaffSymbol": [
        "grob-interface",
        "spanner-interface",
        "staff-symbol-interface",
        ],
    "StanzaNumber": [
        "font-interface",
        "grob-interface",
        "item-interface",
        "side-position-interface",
        "stanza-number-interface",
        "text-interface",
        ],
    "Stem": [
        "grob-interface",
        "item-interface",
        "stem-interface",
        ],
    "StemStub": [
        "grob-interface",
        "item-interface",
        ],
    "StemTremolo": [
        "grob-interface",
        "item-interface",
        "self-alignment-interface",
        "stem-tremolo-interface",
        ],
    "StringNumber": [
        "font-interface",
        "grob-interface",
        "item-interface",
        "number-interface",
        "outside-staff-interface",
        "self-alignment-interface",
        "side-position-interface",
        "string-number-interface",
        "text-interface",
        "text-script-interface",
        ],
    "StrokeFinger": [
        "font-interface",
        "grob-interface",
        "item-interface",
        "outside-staff-interface",
        "self-alignment-interface",
        "side-position-interface",
        "stroke-finger-interface",
        "text-interface",
        "text-script-interface",
        ],
    "SustainPedal": [
        "font-interface",
        "grob-interface",
        "item-interface",
        "piano-pedal-interface",
        "piano-pedal-script-interface",
        "self-alignment-interface",
        "text-interface",
        ],
    "SustainPedalLineSpanner": [
        "axis-group-interface",
        "grob-interface",
        "outside-staff-interface",
        "piano-pedal-interface",
        "side-position-interface",
        "spanner-interface",
        ],
    "System": [
        "axis-group-interface",
        "grob-interface",
        "outside-staff-axis-group-interface",
        "spanner-interface",
        "system-interface",
        ],
    "SystemStartBar": [
        "grob-interface",
        "side-position-interface",
        "spanner-interface",
        "system-start-delimiter-interface",
        ],
    "SystemStartBrace": [
        "font-interface",
        "grob-interface",
        "side-position-interface",
        "spanner-interface",
        "system-start-delimiter-interface",
        ],
    "SystemStartBracket": [
        "font-interface",
        "grob-interface",
        "side-position-interface",
        "spanner-interface",
        "system-start-delimiter-interface",
        ],
    "SystemStartSquare": [
        "font-interface",
        "grob-interface",
        "side-position-interface",
        "spanner-interface",
        "system-start-delimiter-interface",
        ],
    "TabNoteHead": [
        "font-interface",
        "grob-interface",
        "item-interface",
        "note-head-interface",
        "rhythmic-grob-interface",
        "rhythmic-head-interface",
        "staff-symbol-referencer-interface",
        "tab-note-head-interface",
        "text-interface",
        ],
    "TextScript": [
        "font-interface",
        "grob-interface",
        "instrument-specific-markup-interface",
        "item-interface",
        "outside-staff-interface",
        "self-alignment-interface",
        "side-position-interface",
        "text-interface",
        "text-script-interface",
        ],
    "TextSpanner": [
        "font-interface",
        "grob-interface",
        "line-interface",
        "line-spanner-interface",
        "outside-staff-interface",
        "side-position-interface",
        "spanner-interface",
        ],
    "Tie": [
        "grob-interface",
        "spanner-interface",
        "tie-interface",
        ],
    "TieColumn": [
        "grob-interface",
        "spanner-interface",
        "tie-column-interface",
        ],
    "TimeSignature": [
        "break-aligned-interface",
        "font-interface",
        "grob-interface",
        "item-interface",
        "pure-from-neighbor-interface",
        "time-signature-interface",
        ],
    "TrillPitchAccidental": [
        "accidental-interface",
        "font-interface",
        "grob-interface",
        "inline-accidental-interface",
        "item-interface",
        "side-position-interface",
        "trill-pitch-accidental-interface",
        ],
    "TrillPitchGroup": [
        "axis-group-interface",
        "font-interface",
        "grob-interface",
        "item-interface",
        "note-head-interface",
        "parentheses-interface",
        "side-position-interface",
        ],
    "TrillPitchHead": [
        "font-interface",
        "grob-interface",
        "item-interface",
        "ledgered-interface",
        "pitched-trill-interface",
        "rhythmic-head-interface",
        "staff-symbol-referencer-interface",
        ],
    "TrillSpanner": [
        "font-interface",
        "grob-interface",
        "line-interface",
        "line-spanner-interface",
        "outside-staff-interface",
        "side-position-interface",
        "spanner-interface",
        "trill-spanner-interface",
        ],
    "TupletBracket": [
        "grob-interface",
        "line-interface",
        "outside-staff-interface",
        "spanner-interface",
        "tuplet-bracket-interface",
        ],
    "TupletNumber": [
        "font-interface",
        "grob-interface",
        "outside-staff-interface",
        "spanner-interface",
        "text-interface",
        "tuplet-number-interface",
        ],
    "UnaCordaPedal": [
        "font-interface",
        "grob-interface",
        "item-interface",
        "piano-pedal-script-interface",
        "self-alignment-interface",
        "text-interface",
        ],
    "UnaCordaPedalLineSpanner": [
        "axis-group-interface",
        "grob-interface",
        "outside-staff-interface",
        "piano-pedal-interface",
        "side-position-interface",
        "spanner-interface",
        ],
    "VaticanaLigature": [
        "font-interface",
        "grob-interface",
        "spanner-interface",
        "vaticana-ligature-interface",
        ],
    "VerticalAlignment": [
        "align-interface",
        "axis-group-interface",
        "grob-interface",
        "spanner-interface",
        ],
    "VerticalAxisGroup": [
        "axis-group-interface",
        "grob-interface",
        "hara-kiri-group-spanner-interface",
        "outside-staff-axis-group-interface",
        "spanner-interface",
        ],
    "VoiceFollower": [
        "grob-interface",
        "line-interface",
        "line-spanner-interface",
        "spanner-interface",
        ],
    "VoltaBracket": [
        "font-interface",
        "grob-interface",
        "horizontal-bracket-interface",
        "line-interface",
        "side-position-interface",
        "spanner-interface",
        "text-interface",
        "volta-bracket-interface",
        "volta-interface",
        ],
    "VoltaBracketSpanner": [
        "axis-group-interface",
        "grob-interface",
        "outside-staff-interface",
        "side-position-interface",
        "spanner-interface",
        "volta-interface",
        ],
    }



import sys
from .ContextManager import ContextManager


class RedirectedStreams(ContextManager):
    """
    A context manager for capturing stdout and stderr output.

    ..  container:: example

        >>> from io import StringIO
        >>> string_io = StringIO()
        >>> with abjad.RedirectedStreams(stdout=string_io):
        ...     print("hello, world!")
        ...
        >>> result = string_io.getvalue()
        >>> string_io.close()
        >>> print(result)
        hello, world!

    Redirected streams context manager is immutable.
    """

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Context managers'

    __slots__ = (
        '_stdout',
        '_stderr',
        '_old_stderr',
        '_old_stdout',
        )

    ### INITIALIZER ###

    def __init__(self, stdout=None, stderr=None):
        self._stdout = stdout or sys.stdout
        self._stderr = stderr or sys.stderr

    ### SPECIAL METHODS ###

    def __enter__(self):
        """
        Enters redirected streams context manager.

        Returns none.
        """
        self._old_stdout, self._old_stderr = sys.stdout, sys.stderr
        self._old_stdout.flush()
        self._old_stderr.flush()
        sys.stdout, sys.stderr = self._stdout, self._stderr
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        """
        Exits redirected streams context manager.

        Returns none.
        """
        try:
            self._stdout.flush()
            self._stderr.flush()
        except:
            pass
        sys.stdout = self._old_stdout
        sys.stderr = self._old_stderr

    def __repr__(self):
        """
        Gets interpreter representation of context manager.

        ..  container:: example

            >>> context_manager = abjad.RedirectedStreams()
            >>> context_manager
            <RedirectedStreams()>

        Returns string.
        """
        return super().__repr__()

    ### PRIVATE METHODS ###

    def _get_format_specification(self):
        from abjad import system
        return system.FormatSpecification(
            self,
            repr_is_bracketed=True,
            repr_is_indented=False,
            storage_format_is_bracketed=True,
            storage_format_is_indented=False,
            storage_format_args_values=[],
            storage_format_kwargs_names=[],
            )

    ### PUBLIC PROPERTIES ###

    @property
    def stderr(self):
        """
        Gets stderr of context manager.
        """
        return self._stderr

    @property
    def stdout(self):
        """
        Gets stdout of context manager.
        """
        return self._stdout



from abjad.system.FormatSpecification import FormatSpecification
from abjad.system.StorageFormatManager import StorageFormatManager
from abjad.utilities.TypedList import TypedList


class NoteHeadList(TypedList):
    r"""
    Note-head list.

    ..  container:: example

        >>> chord = abjad.Chord([0, 1, 4], (1, 4))
        >>> note_heads = abjad.NoteHeadList(
        ...     client=chord,
        ...     items=[11, 10, 9],
        ...     )

        >>> abjad.f(note_heads)
        abjad.NoteHeadList(
            [
                abjad.NoteHead(
                    written_pitch=abjad.NamedPitch("a'"),
                    ),
                abjad.NoteHead(
                    written_pitch=abjad.NamedPitch("bf'"),
                    ),
                abjad.NoteHead(
                    written_pitch=abjad.NamedPitch("b'"),
                    ),
                ]
            )

    """

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Note-heads'

    __slots__ = (
        '_client',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        items=None,
        client=None,
        ):
        import abjad
        self._client = client
        TypedList.__init__(
            self,
            item_class=abjad.NoteHead,
            keep_sorted=True,
            items=items,
            )

    ### PRIVATE METHODS ###

    def _get_format_specification(self):
        agent = StorageFormatManager(self)
        names = list(agent.signature_keyword_names)
        if 'client' in names:
            names.remove('client')
        if 'items' in names:
            names.remove('items')
        if 'keep_sorted' in names:
            names.remove('keep_sorted')
        return FormatSpecification(
            self,
            repr_is_indented=False,
            storage_format_args_values=[self._collection],
            storage_format_kwargs_names=names,
            )

    def _on_insertion(self, item):
        item._client = self.client

    def _on_removal(self, item):
        item._client = None

    ### PRIVATE PROPERTIES ##

    @property
    def _item_coercer(self):
        def coerce_(token):
            if not isinstance(token, abjad.NoteHead):
                token = abjad.NoteHead(written_pitch=token)
                token._client = self.client
            return token
        import abjad
        return coerce_

    ### PUBLIC METHODS ###

    def extend(self, items):
        r"""
        Extends note-heads.

        ..  container:: example

            >>> chord = abjad.Chord("<ef'>")
            >>> abjad.show(chord) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(chord)
                <ef'>4

            >>> note_heads = []
            >>> note_head = abjad.NoteHead("cs''")
            >>> note_head.tweaks.color = 'blue'
            >>> note_heads.append(note_head)
            >>> note_head = abjad.NoteHead("f''")
            >>> note_head.tweaks.color = 'green'
            >>> note_heads.append(note_head)
            >>> chord.note_heads.extend(note_heads)
            >>> abjad.show(chord) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(chord)
                <
                    ef'
                    \tweak color #blue
                    cs''
                    \tweak color #green
                    f''
                >4

        Returns note-head.
        """
        return super().extend(items)

    def get(self, pitch):
        r"""
        Gets note-head by ``pitch``.

        ..  container:: example

            Gets note-head by pitch name:

            >>> chord = abjad.Chord("<e' cs'' f''>4")
            >>> abjad.show(chord) # doctest: +SKIP

            >>> note_head = chord.note_heads.get("e'")
            >>> note_head.tweaks.color = 'red'
            >>> abjad.show(chord) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(chord)
                <
                    \tweak color #red
                    e'
                    cs''
                    f''
                >4

        ..  container:: example

            Gets note-head by pitch number:

            >>> chord = abjad.Chord("<e' cs'' f''>4")
            >>> abjad.show(chord) # doctest: +SKIP

            >>> note_head = chord.note_heads.get(4)
            >>> note_head.tweaks.color = 'red'
            >>> abjad.show(chord) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(chord)
                <
                    \tweak color #red
                    e'
                    cs''
                    f''
                >4

        Raises missing note-head error when chord contains no
        note-head with ``pitch``.

        Raises extra note-head error when chord contains more than
        one note-head with ``pitch``.

        Returns note-head.
        """
        import abjad
        result = []
        pitch = abjad.NamedPitch(pitch)
        for note_head in self:
            if note_head.written_pitch == pitch:
                result.append(note_head)
        count = len(result)
        if count == 0:
            message = 'missing note-head.'
            raise ValueError(message)
        elif count == 1:
            note_head = result[0]
            return note_head
        else:
            message = 'extra note-head.'
            raise ValueError(message)

    def pop(self, i=-1):
        r"""
        Pops note-head ``i``.

        ..  container:: example

            >>> chord = abjad.Chord("<ef' c'' f''>4")
            >>> abjad.show(chord) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(chord)
                <ef' c'' f''>4

            >>> chord.note_heads.pop(1)
            NoteHead("c''")

            >>> abjad.show(chord) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(chord)
                <ef' f''>4

        Returns note-head.
        """
        return super().pop(i=i)

    def remove(self, item):
        r"""
        Removes ``item``.

        ..  container:: example

            >>> chord = abjad.Chord("<ef' c'' f''>4")
            >>> abjad.show(chord) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(chord)
                <ef' c'' f''>4

            >>> note_head = chord.note_heads[1]
            >>> chord.note_heads.remove(note_head)
            >>> abjad.show(chord) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(chord)
                <ef' f''>4

        """
        return super().remove(item)

    ### PUBLIC PROPERTIES ###

    @property
    def client(self):
        """
        Gets client.
        """
        return self._client



import typing
from abjad.system.AbjadValueObject import AbjadValueObject
from abjad.scheme import Scheme
from abjad.system.LilyPondFormatBundle import LilyPondFormatBundle


class LilyPondGrobOverride(AbjadValueObject):
    r"""
    LilyPond grob override.

    ..  container:: example

        >>> override = abjad.LilyPondGrobOverride(
        ...    lilypond_type='Staff',
        ...    grob_name='TextSpanner',
        ...    once=True,
        ...    property_path=(
        ...        'bound-details',
        ...        'left',
        ...        'text',
        ...        ),
        ...    value=abjad.Markup(r'\bold { over pressure }'),
        ...    )

        >>> print(override.override_string)
        \once \override Staff.TextSpanner.bound-details.left.text = \markup {
            \bold
                {
                    over
                    pressure
                }
            }

    """

    ### CLASS VARIABLES ###

    __slots__ = (
        '_grob_name',
        '_is_revert',
        '_lilypond_type',
        '_once',
        '_property_path',
        '_value',
        )

    _format_leaf_children = False

    ### INITIALIZER ###

    def __init__(
        self,
        lilypond_type: str = None,
        grob_name: str = 'NoteHead',
        once: bool = None,
        is_revert: bool = None,
        property_path: typing.Union[str, typing.Iterable[str]] = 'color',
        value: typing.Any = 'red',
        ) -> None:
        if lilypond_type is not None:
            lilypond_type = str(lilypond_type)
        self._lilypond_type = lilypond_type
        assert grob_name
        self._grob_name = str(grob_name)
        if once is not None:
            once = bool(once)
        self._once = once
        if is_revert is not None:
            is_revert = bool(is_revert)
        self._is_revert = is_revert
        if isinstance(property_path, str):
            property_path_: typing.Tuple[str, ...] = (property_path,)
        else:
            property_path_ = tuple(property_path)
        assert isinstance(property_path_, tuple), repr(property_path_)
        assert all(isinstance(_, str) for _ in property_path_)
        assert all(_ != '' for _ in property_path_)
        self._property_path = property_path_
        self._value = value

    ### SPECIAL METHODS ###

    def __eq__(self, argument) -> bool:
        """
        Is true when ``argument`` is a LilyPond grob override with equivalent
        keyword values.
        """
        return super().__eq__(argument)

    def __hash__(self) -> int:
        """
        Hashes LilyPond grob override.

        Redefined in tandem with __eq__.
        """
        return super().__hash__()

    ### PRIVATE METHODS ###

    def _get_lilypond_format_bundle(self, component=None):
        bundle = LilyPondFormatBundle()
        if not self.once:
            revert_format = '\n'.join(self.revert_format_pieces)
            bundle.grob_reverts.append(revert_format)
        if not self.is_revert:
            override_format = '\n'.join(self.override_format_pieces)
            bundle.grob_overrides.append(override_format)
        return bundle

    def _override_property_path_string(self):
        parts = []
        if self.lilypond_type is not None:
            parts.append(self.lilypond_type)
        parts.append(self.grob_name)
        parts.extend(self.property_path)
        path = '.'.join(parts)
        return path

    def _revert_property_path_string(self):
        parts = []
        if self.lilypond_type is not None:
            parts.append(self.lilypond_type)
        parts.append(self.grob_name)
        parts.append(self.property_path[0])
        path = '.'.join(parts)
        return path

    ### PUBLIC PROPERTIES ###

    @property
    def lilypond_type(self) -> typing.Optional[str]:
        r"""
        Gets LilyPond type of context.

        ..  container:: example

            >>> override = abjad.LilyPondGrobOverride(
            ...    lilypond_type='Staff',
            ...    grob_name='TextSpanner',
            ...    once=True,
            ...    property_path=(
            ...        'bound-details',
            ...        'left',
            ...        'text',
            ...        ),
            ...    value=abjad.Markup(r'\bold { over pressure }'),
            ...    )
            >>> override.lilypond_type
            'Staff'

            >>> override = abjad.LilyPondGrobOverride(
            ...     grob_name='Glissando',
            ...     property_path='style',
            ...     value=abjad.SchemeSymbol('zigzag'),
            ...     )
            >>> override.lilypond_type is None
            True

        """
        return self._lilypond_type

    @property
    def grob_name(self) -> str:
        r"""
        Gets grob name.

        ..  container:: example

            >>> override = abjad.LilyPondGrobOverride(
            ...     grob_name='Glissando',
            ...     property_path='style',
            ...     value=abjad.SchemeSymbol('zigzag'),
            ...     )
            >>> override.grob_name
            'Glissando'

        """
        return self._grob_name

    @property
    def once(self) -> typing.Optional[bool]:
        r"""
        Is true when grob override is to be applied only once.

        ..  container:: example

            >>> override = abjad.LilyPondGrobOverride(
            ...    lilypond_type='Staff',
            ...    grob_name='TextSpanner',
            ...    once=True,
            ...    property_path=(
            ...        'bound-details',
            ...        'left',
            ...        'text',
            ...        ),
            ...    value=abjad.Markup(r'\bold { over pressure }'),
            ...    )
            >>> bool(override.once)
            True

            >>> override = abjad.LilyPondGrobOverride(
            ...     grob_name='Glissando',
            ...     property_path='style',
            ...     value=abjad.SchemeSymbol('zigzag'),
            ...     )
            >>> bool(override.once)
            False

        """
        return self._once

    @property
    def is_revert(self) -> typing.Optional[bool]:
        r"""
        Is true if grob override is a grob revert.

        ..  container:: example

            >>> override = abjad.LilyPondGrobOverride(
            ...     grob_name='Glissando',
            ...     property_path='style',
            ...     value=abjad.SchemeSymbol('zigzag'),
            ...     )
            >>> bool(override.is_revert)
            False

            >>> override = abjad.LilyPondGrobOverride(
            ...     grob_name='Glissando',
            ...     is_revert=True,
            ...     property_path='style',
            ...     )
            >>> bool(override.is_revert)
            True

        """
        return self._is_revert

    @property
    def override_format_pieces(self) -> typing.Tuple[str, ...]:
        r"""
        Gets LilyPond grob override \override format pieces.

        ..  container:: example

            >>> override = abjad.LilyPondGrobOverride(
            ...    lilypond_type='Staff',
            ...    grob_name='TextSpanner',
            ...    once=True,
            ...    property_path=(
            ...        'bound-details',
            ...        'left',
            ...        'text',
            ...        ),
            ...    value=abjad.Markup(r'\bold { over pressure }'),
            ...    )
            >>> for line in override.override_format_pieces:
            ...     line
            ...
            '\\once \\override Staff.TextSpanner.bound-details.left.text = \\markup {'
            '    \\bold'
            '        {'
            '            over'
            '            pressure'
            '        }'
            '    }'

        """
        result = []
        if self.once:
            result.append(r'\once')
        result.append(r'\override')
        result.append(self._override_property_path_string())
        result.append('=')
        string = Scheme.format_embedded_scheme_value(self.value)
        value_pieces = string.split('\n')
        result.append(value_pieces[0])
        result[:] = [' '.join(result)]
        result.extend(value_pieces[1:])
        return tuple(result)

    @property
    def override_string(self) -> str:
        r"""
        Gets LilyPond grob override \override string.

        ..  container:: example

            >>> override = abjad.LilyPondGrobOverride(
            ...     grob_name='Glissando',
            ...     property_path='style',
            ...     value=abjad.SchemeSymbol('zigzag'),
            ...     )
            >>> override.override_string
            "\\override Glissando.style = #'zigzag"

        """
        return '\n'.join(self.override_format_pieces)

    @property
    def property_path(self) -> typing.Tuple[str, ...]:
        r"""
        Gets LilyPond grob override property path.

        ..  container:: example

            >>> override = abjad.LilyPondGrobOverride(
            ...    lilypond_type='Staff',
            ...    grob_name='TextSpanner',
            ...    once=True,
            ...    property_path=(
            ...        'bound-details',
            ...        'left',
            ...        'text',
            ...        ),
            ...    value=abjad.Markup(r'\bold { over pressure }'),
            ...    )
            >>> override.property_path
            ('bound-details', 'left', 'text')

        """
        return self._property_path

    @property
    def revert_format_pieces(self) -> typing.Tuple[str, ...]:
        r"""
        Gets LilyPond grob override \revert format pieces.

        ..  container:: example

            >>> override = abjad.LilyPondGrobOverride(
            ...     grob_name='Glissando',
            ...     property_path='style',
            ...     value=abjad.SchemeSymbol('zigzag'),
            ...     )
            >>> override.revert_format_pieces
            ('\\revert Glissando.style',)

        """
        result = rf'\revert {self._revert_property_path_string()}'
        return (result,)

    @property
    def revert_string(self) -> str:
        r"""
        Gets LilyPond grob override \revert string.

        ..  container:: example

            >>> override = abjad.LilyPondGrobOverride(
            ...     grob_name='Glissando',
            ...     property_path='style',
            ...     value=abjad.SchemeSymbol('zigzag'),
            ...     )
            >>> override.revert_string
            '\\revert Glissando.style'

        """
        return '\n'.join(self.revert_format_pieces)

    @property
    def value(self) -> typing.Any:
        r"""
        Gets value of LilyPond grob override.

        ..  container:: example

            >>> override = abjad.LilyPondGrobOverride(
            ...    lilypond_type='Staff',
            ...    grob_name='TextSpanner',
            ...    once=True,
            ...    property_path=(
            ...        'bound-details',
            ...        'left',
            ...        'text',
            ...        ),
            ...    value=abjad.Markup(r'\bold { over pressure }'),
            ...    )
            >>> override.value
            Markup(contents=[MarkupCommand('bold', ['over', 'pressure'])])

        """
        return self._value

    ### PUBLIC METHODS ###

    def tweak_string(self, directed=True, grob=False) -> str:
        r"""
        Gets LilyPond grob override \tweak string.

        ..  container:: example

            >>> override = abjad.LilyPondGrobOverride(
            ...     grob_name='Glissando',
            ...     property_path='style',
            ...     value=abjad.SchemeSymbol('zigzag'),
            ...     )
            >>> override.tweak_string()
            "- \\tweak style #'zigzag"

        ..  container:: example

            >>> override = abjad.LilyPondGrobOverride(
            ...     grob_name='RehearsalMark',
            ...     property_path='color',
            ...     value='red',
            ...     )
            >>> override.tweak_string(directed=False)
            '\\tweak color #red'

        ..  container:: example

            LilyPond literals are allowed:

            >>> override = abjad.LilyPondGrobOverride(
            ...     grob_name='TextSpann',
            ...     property_path=('bound-details', 'left-broken', 'text'),
            ...     value=abjad.LilyPondLiteral(r'\markup \upright pont.'),
            ...     )
            >>> override.tweak_string(directed=False)
            '\\tweak bound-details.left-broken.text \\markup \\upright pont.'

        """
        from abjad.indicators.LilyPondLiteral import LilyPondLiteral
        if directed:
            result = [r'- \tweak']
        else:
            result = [r'\tweak']
        if grob:
            property_path = (self.grob_name,) + self.property_path
        else:
            property_path = self.property_path
        string = '.'.join(property_path)
        result.append(string)
        if isinstance(self.value, LilyPondLiteral):
            assert isinstance(self.value.argument, str)
            string = self.value.argument
        else:
            string = Scheme.format_embedded_scheme_value(self.value)
        result.append(string)
        return ' '.join(result)



"""
Tools for modeling "timespans".
"""

from .Timespan import Timespan
from .AnnotatedTimespan import AnnotatedTimespan
from .CompoundInequality import CompoundInequality
from .TimeRelation import TimeRelation
from .OffsetTimespanTimeRelation import OffsetTimespanTimeRelation
from .TimespanInequality import TimespanInequality
from .TimespanList import TimespanList
from .TimespanTimespanTimeRelation import TimespanTimespanTimeRelation
from .offset_happens_after_timespan_starts import offset_happens_after_timespan_starts
from .offset_happens_after_timespan_stops import offset_happens_after_timespan_stops
from .offset_happens_before_timespan_starts import offset_happens_before_timespan_starts
from .offset_happens_before_timespan_stops import offset_happens_before_timespan_stops
from .offset_happens_during_timespan import offset_happens_during_timespan
from .offset_happens_when_timespan_starts import offset_happens_when_timespan_starts
from .offset_happens_when_timespan_stops import offset_happens_when_timespan_stops
from .timespan_2_contains_timespan_1_improperly import timespan_2_contains_timespan_1_improperly
from .timespan_2_curtails_timespan_1 import timespan_2_curtails_timespan_1
from .timespan_2_delays_timespan_1 import timespan_2_delays_timespan_1
from .timespan_2_happens_during_timespan_1 import timespan_2_happens_during_timespan_1
from .timespan_2_intersects_timespan_1 import timespan_2_intersects_timespan_1
from .timespan_2_is_congruent_to_timespan_1 import timespan_2_is_congruent_to_timespan_1
from .timespan_2_overlaps_all_of_timespan_1 import timespan_2_overlaps_all_of_timespan_1
from .timespan_2_overlaps_only_start_of_timespan_1 import timespan_2_overlaps_only_start_of_timespan_1
from .timespan_2_overlaps_only_stop_of_timespan_1 import timespan_2_overlaps_only_stop_of_timespan_1
from .timespan_2_overlaps_start_of_timespan_1 import timespan_2_overlaps_start_of_timespan_1
from .timespan_2_overlaps_stop_of_timespan_1 import timespan_2_overlaps_stop_of_timespan_1
from .timespan_2_starts_after_timespan_1_starts import timespan_2_starts_after_timespan_1_starts
from .timespan_2_starts_after_timespan_1_stops import timespan_2_starts_after_timespan_1_stops
from .timespan_2_starts_before_timespan_1_starts import timespan_2_starts_before_timespan_1_starts
from .timespan_2_starts_before_timespan_1_stops import timespan_2_starts_before_timespan_1_stops
from .timespan_2_starts_during_timespan_1 import timespan_2_starts_during_timespan_1
from .timespan_2_starts_when_timespan_1_starts import timespan_2_starts_when_timespan_1_starts
from .timespan_2_starts_when_timespan_1_stops import timespan_2_starts_when_timespan_1_stops
from .timespan_2_stops_after_timespan_1_starts import timespan_2_stops_after_timespan_1_starts
from .timespan_2_stops_after_timespan_1_stops import timespan_2_stops_after_timespan_1_stops
from .timespan_2_stops_before_timespan_1_starts import timespan_2_stops_before_timespan_1_starts
from .timespan_2_stops_before_timespan_1_stops import timespan_2_stops_before_timespan_1_stops
from .timespan_2_stops_during_timespan_1 import timespan_2_stops_during_timespan_1
from .timespan_2_stops_when_timespan_1_starts import timespan_2_stops_when_timespan_1_starts
from .timespan_2_stops_when_timespan_1_stops import timespan_2_stops_when_timespan_1_stops
from .timespan_2_trisects_timespan_1 import timespan_2_trisects_timespan_1



def is_positive_integer_power_of_two(argument):
    r"""
    Is true when ``argument`` is a positive integer power of 2.

    ..  container:: example

        >>> for n in range(10):
        ...     print(n, abjad.mathtools.is_positive_integer_power_of_two(n))
        ...
        0 False
        1 True
        2 True
        3 False
        4 True
        5 False
        6 False
        7 False
        8 True
        9 False

    Returns true or false.
    """
    from abjad import mathtools
    return (
        0 < argument and
        mathtools.is_nonnegative_integer_power_of_two(argument)
        )



from abjad import markups
from abjad.system.AbjadValueObject import AbjadValueObject


class Retrograde(AbjadValueObject):
    """
    Retrograde operator.

    ..  container:: example:

        >>> abjad.Retrograde()
        Retrograde()

    Object model of twelve-tone retrograde operator.
    """

    ### CLASS VARIABLES ###

    __slots__ = (
        '_period',
        )

    ### INITIALIZER ###

    def __init__(self, period=None):
        if period is not None:
            period = abs(int(period))
            assert 0 < period
        self._period = period

    ### SPECIAL METHODS ###

    def __add__(self, operator):
        r"""
        Composes retrograde and `operator`.

        ..  container:: example

            Example segment:

            >>> items = [0, 2, 4, 5]
            >>> segment = abjad.PitchClassSegment(items=items)
            >>> abjad.show(segment) # doctest: +SKIP

            Example operators:

            >>> retrograde = abjad.Retrograde()
            >>> transposition = abjad.Transposition(n=3)

        ..  container:: example

            Transposition followed by retrograde:

            >>> operator = retrograde + transposition
            >>> str(operator)
            'RT3'

            >>> segment_ = operator(segment)
            >>> abjad.show(segment_) # doctest: +SKIP

            ..  docs::

                >>> lilypond_file = segment_.__illustrate__()
                >>> abjad.f(lilypond_file[abjad.Voice])
                \new Voice
                {
                    af'8
                    g'8
                    f'8
                    ef'8
                    \bar "|." %! SCORE1
                    \override Score.BarLine.transparent = ##f
                }

        ..  container:: example

            Same as above because retrograde and transposition commute:

            >>> operator = transposition + retrograde
            >>> str(operator)
            'T3R'

            >>> segment_ = operator(segment)
            >>> abjad.show(segment_) # doctest: +SKIP

            ..  docs::

                >>> lilypond_file = segment_.__illustrate__()
                >>> abjad.f(lilypond_file[abjad.Voice])
                \new Voice
                {
                    af'8
                    g'8
                    f'8
                    ef'8
                    \bar "|." %! SCORE1
                    \override Score.BarLine.transparent = ##f
                }

        ..  container:: example

            Returns compound operator:

            >>> abjad.f(operator)
            abjad.CompoundOperator(
                operators=[
                    abjad.Retrograde(),
                    abjad.Transposition(
                        n=3,
                        ),
                    ],
                )

        """
        import abjad
        return abjad.CompoundOperator._compose_operators(self, operator)

    def __call__(self, argument):
        """
        Calls retrograde on `argument`.

        ..  container:: example

            Gets retrograde pitch classes:

            >>> retrograde = abjad.Retrograde()
            >>> segment = abjad.PitchClassSegment([0, 1, 4, 7])
            >>> retrograde(segment)
            PitchClassSegment([7, 4, 1, 0])

        ..  container:: example

            Does not retrograde single pitches or pitch-classes:

            >>> retrogresion = abjad.Retrograde()
            >>> pitch_class = abjad.NumberedPitchClass(6)
            >>> retrograde(pitch_class)
            NumberedPitchClass(6)

        ..  container:: example

            Periodic retrograde:

            ..  todo:: Deprecated.

            >>> retrograde = abjad.Retrograde(period=3)
            >>> segment = abjad.PitchSegment("c' d' e' f' g' a' b' c''")
            >>> retrograde(segment)
            PitchSegment("e' d' c' a' g' f' c'' b'")

        Returns new object with type equal to that of `argument`.
        """
        import abjad
        if isinstance(argument, (abjad.Pitch, abjad.PitchClass)):
            return argument
        if not isinstance(argument, (
            abjad.PitchSegment, abjad.PitchClassSegment,
            )):
            argument = abjad.PitchSegment(argument)
        if not self.period:
            return type(argument)(reversed(argument))
        result = abjad.new(argument, items=())
        for shard in abjad.sequence(argument).partition_by_counts(
            [self.period],
            cyclic=True,
            overhang=True,
            ):
            shard = type(argument)(shard)
            shard = type(argument)(reversed(shard))
            result = result + shard
        return result

    def __radd__(self, operator):
        """
        Right-addition not defined on retrograde.

        ..  container:: example

            >>> abjad.Retrograde().__radd__(abjad.Retrograde())
            Traceback (most recent call last):
                ...
            NotImplementedError: right-addition not defined on Retrograde.

        Raises not implemented error.
        """
        message = 'right-addition not defined on {}.'
        message = message.format(type(self).__name__)
        raise NotImplementedError(message)

    def __str__(self):
        """
        Gets string representation of operator.

        ..  container:: example

            >>> str(abjad.Retrograde())
            'R'

        """
        return 'R'

    ### PRIVATE METHODS ###

    def _get_markup(self, direction=None):
        return markups.Markup('R', direction=direction)

    def _is_identity_operator(self):
        return False

    ### PUBLIC PROPERTIES ###

    @property
    def period(self):
        """
        Gets optional period of retrograde.

        ..  todo:: Deprecated. Use Expression followed by Retrograde instead.

        ..  container:: example

            >>> retrograde = abjad.Retrograde(period=3)
            >>> retrograde.period
            3

        Returns integer or none.
        """
        return self._period



import sys
from .ContextManager import ContextManager


class ProgressIndicator(ContextManager):
    """
    A context manager for printing progress indications.
    """

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Context managers'

    __slots__ = (
        '_is_warning',
        '_message',
        '_progress',
        '_total',
        '_verbose',
        )

    RED = '\033[91m'
    END = '\033[0m'

    ### INITIALIZER ###

    def __init__(
        self,
        message='',
        total=None,
        verbose=True,
        is_warning=None,
        ):
        self._message = message
        self._progress = 0
        self._total = total
        self._verbose = bool(verbose)
        self._is_warning = bool(is_warning)

    ### SPECIAL METHODS ###

    def __enter__(self):
        """
        Enters progress indicator.
        """
        self._print()
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        """
        Exits progress indicator.
        """
        if self.verbose:
            print()

    def __repr__(self):
        """
        Gets interpreter representation of context manager.

        ..  container:: example

            >>> context_manager = abjad.ProgressIndicator()
            >>> context_manager
            <ProgressIndicator()>

        Returns string.
        """
        return '<{}()>'.format(type(self).__name__)

    ### PRIVATE METHODS ###

    def _print(self):
        if not self.verbose:
            return
        message = self.message or 'Progress'
        if self.total is not None:
            message = '{}: {} / {}'.format(
                message,
                self.progress,
                self.total,
                )
        else:
            message = '{}: {}'.format(
                message,
                self.progress,
                )
        if self.is_warning and self.progress:
            message = self.RED + message + self.END
        print(message, end='')

    ### PUBLIC METHODS ###

    def advance(self):
        """
        Advances the progress indicator's progress count.  Overwrites
        the current terminal line with the progress indicators message and new
        count.
        """
        self._progress += 1
        if self.verbose:
            sys.stdout.flush()
            print('\r', end='')
        self._print()

    ### PUBLIC PROPERTIES ###

    @property
    def is_warning(self):
        """
        Is true if progress indicator prints in red when its progress goes
        above zero.

        Returns true or false.
        """
        return self._is_warning

    @property
    def message(self):
        """
        Gets message of progress indicator.

        Returns string.
        """
        return self._message

    @property
    def progress(self):
        """
        Gets progress.

        Returns integer.
        """
        return self._progress

    @property
    def total(self):
        """
        Gets total count.

        Returns integer or none.
        """
        return self._total

    @property
    def verbose(self):
        """
        Is true if progress indicator prints status.

        Returns true or false.
        """
        return self._verbose



def is_integer_equivalent_n_tuple(argument, n):
    """
    Is true when ``argument`` is a tuple of ``n`` integer-equivalent items.

    ..  container:: example

        >>> tuple_ = (2.0, '3', abjad.Fraction(4, 1))
        >>> abjad.mathtools.is_integer_equivalent_n_tuple(tuple_, 3)
        True

        >>> tuple_ = (2.5, '3', abjad.Fraction(4, 1))
        >>> abjad.mathtools.is_integer_equivalent_n_tuple(tuple_, 3)
        False

    Returns true or false.
    """
    from abjad import mathtools
    return (
        isinstance(argument, tuple) and
        len(argument) == n and
        all(mathtools.is_integer_equivalent(_) for _ in argument)
        )
